
/*  avwsplit.c - split 4D Analyze file into 3D files for SPM

    David Flitney, FMRIB Image Analysis Group

    Copyright (C) 2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <avwio.h>

int main(int argc, char **argv)
{
  AVW *src;
  short x, y, z, v, t;
  float vx, vy, vz, tr;
  size_t bufsz;
  void *buffer;
  unsigned int i;
  char units[1024];

   if (argc<2)
    {
      printf("\nUsage: avwsplit <inputfile.hdr>\n\n");
      exit(1);
    }

  if((src=AvwOpen(argv[1],"r"))==NULL) {
    perror("avwsplit");
    exit(-1);
  }

  AvwGetDim(src,&x,&y,&z,&v);
  AvwGetVoxDim(src,&vx,&vy,&vz,&tr);
  AvwGetVoxUnits(src, units);

  bufsz = x * y * z * (AvwGetDataType(src, &t) / 8);
  buffer = (void *) calloc(bufsz, sizeof(char));

  for(i = 0; i < v; i++) {
    AVW *dest;
    char opfn[1024];

    AvwReadVolumes(src, buffer, 1);

    sprintf(opfn, "vol%04d", i);
    fprintf(stderr, "Writing %s\r", opfn);

    dest = AvwOpen(opfn, "w");
    AvwCloneHeader(dest, src);
    AvwSetDim(dest, x, y, z, 1);
    AvwWriteVolumes(dest, buffer, 1);
    AvwClose(dest);
  }

  fprintf(stderr, "\n");

  return(0);
}
