#{{{ copyright and setup

#   FEAT - FMRIB's Easy Analysis Tool for FMRI
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2001 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

source [ file dirname [ info script ] ]/fslstart.tcl

set VARS(history) {}

set VERSION 4.03

#}}}
#{{{ feat4 (main GUI)

proc feat4 { w } {
    global fmri PXHOME FSLDIR USER VERSION INMEDX feat_files highres_files VARS argc argv PWD
 
    #{{{ setup main window

toplevel $w

set fmri(version) $VERSION

set fmri(feat_filename) [ exec ${FSLDIR}/bin/tmpnam /tmp/feat ].fsf

wm title      $w "FEAT - FMRIB's Easy Analysis Tool - $fmri(version)"
wm iconname   $w "FEAT"
wm iconbitmap $w @${FSLDIR}/tcl/fmrib.xbm

tixBalloon $w.bhelp -initwait 5000 -options { background #bfdfff }

frame $w.f
pack $w.f -in $w -side top

#}}}
    #{{{ Data

tixLabelFrame $w.f.data -label "Data"
set fmri(lfdata) [ $w.f.data subwidget frame ]

frame $w.f.datamain

#{{{ multiple analyses

frame $w.f.multiple

set fmri(multiple) 1

tixControl $w.f.multiple.number -label "First-level analyses " -padx 2 \
	-variable fmri(multiple) -step 1 -min 1 -selectmode immediate \
	-options { entry.width 2 }

button $w.f.multiple.setup -text "Select 4D data" -command "feat4:multiple_select $w"

pack $w.f.multiple.number $w.f.multiple.setup -in $w.f.multiple -side left -padx 5

set medxmessage ""
if { $INMEDX } {
    $w.f.multiple.number configure -min 0 -label "File-based first-level analyses "
    if { [ what_type 0 ] == 4 } {
	MxGetCurrentPage currentpage
	MxGetPageProperties $currentpage Props
	set fmri(groupname) [ keylget Props Name ]
	set fmri(multiple) 0
    }

    label $w.f.groupname

    set medxmessage "
If the number of \"File-based first-level analyses\" is set to 0
then FEAT will use the selected group page inside the current MEDx
folder for analysis, instead of a file on disk.
"
}

$w.bhelp bind $w.f.multiple -msg "Set the filename of the 4D input image (e.g. /home/sibelius/func.hdr).
You can setup FEAT to process many input images, one after another, as
long as they all require exactly the same analysis. Each one will
generate its own FEAT directory, the name of which is based on the
input data's filename. Note that if you are going to re-run later
parts of an analysis (or run group stats separately from the
first-level analyses) the selection of 4D data changes to the
selection of FEAT directories. In this case first change the \"Full
first-level analysis\" option to whatever is appropriate
(e.g. \"Contrasts, Thresholding, Rendering\") inside \"Advanced
options\" and then select the FEAT directory or directories.
$medxmessage
Note that in this case you should select the FEAT directories before
setting up anything else in FEAT (such as changing the
thresholds). This is because quite a lot of FEAT settings are loaded
from the first selected FEAT directory, possibly over-writing any
settings which you wish to change!"

#}}}
#{{{ npts & ndelete

frame $w.f.nptsndelete

#{{{ npts

set fmri(npts) 0

label $w.f.npts -text " Total volumes $fmri(npts)"

$w.bhelp bind $w.f.npts -msg "The number of FMRI volumes in the time series, including any initial
volumes that you wish to delete. This will get set automatically once
valid input data has been selected."

#}}}
#{{{ ndelete

set fmri(ndelete) 0

tixControl $w.f.ndelete -label "     Delete volumes" -variable fmri(ndelete) \
	-step 1 -min 0 -selectmode immediate -options { entry.width 3 }

$w.bhelp bind $w.f.ndelete -msg "The number of initial FMRI volumes to delete before any further
processing. These should be the volumes that are not wanted because
steady-state imaging has not yet been reached - typically one or two
volumes. These volumes are deleted as soon as FEAT is started, so all
4D data sets produced by FEAT will not contain the deleted volumes.

Note that \"Delete volumes\" should not be used to correct for the
time lag between stimulation and the measured response - this is
corrected for in the design matrix by convolving the input stimulation
waveform with a blurring-and-delaying haemodynamic response function.

Most importantly, remember when setting up the design matrix, that the
timings in the design matrix start at t=0 seconds, and this
corresponds to the start of the first image taken after the deleted
scans. In other words, the design matrix starts AFTER the \"deleted
scans\" have been deleted."

#}}}

pack $w.f.npts $w.f.ndelete -in $w.f.nptsndelete -side left -pady 3

#}}}
#{{{ TR & highpass

frame $w.f.trparadigm_hp

#{{{ TR

set fmri(tr) 3.0
tixControl $w.f.tr -label " TR (s)" -variable fmri(tr) \
	-step 0.25 -min 0.001 -selectmode immediate -options { entry.width 4 }
$w.bhelp bind $w.f.tr -msg "The time (in seconds) between scanning successive FMRI volumes."

#}}}
#{{{ High pass

set fmri(paradigm_hp) 100
tixControl $w.f.paradigm_hp -label "    High pass filter cutoff (s)" \
	-variable fmri(paradigm_hp) -step 5 -min 1 -selectmode immediate -options { entry.width 5 }

$w.bhelp bind $w.f.paradigm_hp -msg "The high pass frequency cutoff point (seconds), that is, the longest
temporal period that you will allow. A sensible setting in the case of
an ABAB or ABACABAC type block design is 1.5 * the (A+B) or (A+B+A+C)
total cycle time."

#}}}

pack $w.f.tr $w.f.paradigm_hp -in $w.f.trparadigm_hp -side left -pady 3

#}}}

pack $w.f.nptsndelete $w.f.trparadigm_hp -in $w.f.datamain -anchor w
pack $w.f.multiple $w.f.datamain -in $fmri(lfdata) -anchor w
pack $w.f.data -in $w.f -anchor w -pady 4

#}}}
    #{{{ Stats

tixLabelFrame $w.f.paradigm -label "Stats"
set fmri(lfparadigm) [$w.f.paradigm subwidget frame]

set fmri(stats_yn) 1

frame $w.f.paradigmmain
frame $w.f.paradigmmainalt

#{{{ general linear model

frame $w.f.ud

button $w.f.wizard -text "Simple model setup" -command "feat4:film_wizard $w"
$w.bhelp bind $w.f.wizard -msg "This lets you easily setup either ABAB... or ABACABAC... designs
(block or single-event).

If you need to adjust the resulting setup, use \"Simple model setup\"
first, then press the \"Full model setup\" button to allow more
detailed setup of the model-based analysis."

button $w.f.model -text "Full model setup" -command "feat4:setup_model $w"
$w.bhelp bind $w.f.model -msg "This allows complete control of the model-based analysis to be used."

button $w.f.modelcon -text "Edit contrasts" -command "feat4:setup_model $w"
pack $w.f.modelcon -in $w.f.paradigmmainalt -anchor w -padx 3
$w.bhelp bind $w.f.modelcon -msg "This allows setup of contrasts and F-tests, to be run on a previous
analysis."

pack $w.f.wizard $w.f.model -in $w.f.ud -side left -padx 5 -pady 3

set fmri(w_model) 0
set fmri(constcol) 0
set fmri(wizard_type) 1
set fmri(a_count) 30
set fmri(b_count) 30
feat4:setup_model_vars_simple $w
set fmri(paradigm_hp) 100
set fmri(filmsetup) 0

#}}}
#{{{ frame irva

frame $w.f.mf

set fmri(mf_type) 1	
radiobutton $w.f.mf_cyclic    -text "Cyclic"       -value 1 -variable fmri(mf_type) -command "feat4:updatemf $w 0"
radiobutton $w.f.mf_triggered -text "Triggered   " -value 2 -variable fmri(mf_type) -command "feat4:updatemf $w 0"
set mf_type_text "Select whether to use a cyclic IRVA analysis (normally for
\"block-like\" experiments) or triggered analysis (normally for
irregularly-spaced event-related experiments)."
$w.bhelp bind $w.f.mf_cyclic -msg $mf_type_text
$w.bhelp bind $w.f.mf_triggered -msg $mf_type_text

set fmri(mf_cycle) 20
tixControl $w.f.mf_cycle -label "Volumes for complete cycle" \
	-command "feat4:updatemf $w" \
	-variable fmri(mf_cycle) -step 1 -min 1 -selectmode immediate -options { entry.width 4 }
$w.bhelp bind $w.f.mf_cycle -msg "The period (volumes) of each stimulation cycle (e.g., for an ABAB
design, this is A+B)."

tixControl $w.f.mf_vols -label "Volumes per event" \
	-command "feat4:updatemf $w" \
	-variable fmri(mf_cycle) -step 1 -min 1 -selectmode immediate -options { entry.width 4 }
$w.bhelp bind $w.f.mf_vols -msg "The number of volumes to include in analysis after each trigger in the
event file."

set fmri(mf_file) "~"
FSLFileEntry $w.f.mf_file \
	-variable fmri(mf_file) \
	-pattern "*" \
	-directory $PWD \
	-label "  File of events" \
	-title "Select an event file" \
	-width 15 \
	-filterhist VARS(history)
$w.bhelp bind $w.f.mf_file -msg "Enter the name of a text file containing the volume number (first
volume is 0) of the start of each event. Use a separate line for each
number."

pack $w.f.mf_cyclic $w.f.mf_triggered $w.f.mf_cycle -in $w.f.mf -side left -padx 5 -pady 3

#}}}
#{{{ paradigm type selection

global fmri(paradigm_type)
set fmri(paradigm_type) 3

tixOptionMenu $w.f.paradigm_type -variable fmri(paradigm_type) -command "feat4:updateparadigm $w 0 1"
$w.f.paradigm_type add command 3 -label "FILM model-based"
$w.f.paradigm_type add command 4 -label "IRVA semi-model-free"

pack $w.f.paradigm_type -in $w.f.paradigmmain -anchor w -padx 3
pack $w.f.paradigmmain -in $fmri(lfparadigm) -anchor w

feat4:updateparadigm $w 0 1 -1

$w.bhelp bind $w.f.paradigm_type -msg "\"FILM\" model-based (FMRIB's Improved Linear Model) uses a
sophisticated implementation of the General Linear Model (GLM),
including pre-whitening of residuals for maximal efficiency and
validity. This should be used for block or single-event designs when
enough is understood about the expected responses for a full
model-based analysis to be sensible.

\"IRVA\" semi-model-free (Inter-Repetition Variance Analysis)
processes data for which the expected response to the stimulus is not
well-known in advance, as long as the timings of each repetition of
the stimulus are known."

#}}}

pack $w.f.paradigm -in $w.f -anchor w -pady 4

#}}}
    #{{{ Advanced options

collapsible frame $w.f.opts -title "Advanced options"
pack $w.f.opts -in $w.f -anchor w -pady 4

#{{{ option menus

frame $w.f.opts.metaopts

#{{{ Best/Fastest

set fmri(type) 2
tixOptionMenu $w.f.opts.metaopts.type -variable fmri(type) -command "feat4:updatetype $w 1"
$w.f.opts.metaopts.type add command 2 -label "Best"
$w.f.opts.metaopts.type add command 1 -label "Fastest"
$w.bhelp bind $w.f.opts.metaopts.type -msg "Choose whether you want FEAT to be as fast as possible or give the
best possible results."

#}}}
#{{{ Full/part first-level analysis

global fmri(analysis)
tixOptionMenu $w.f.opts.metaopts.analysis -variable fmri(analysis)
$w.f.opts.metaopts.analysis add command 0 -label "No first-level analysis (registration and/or group stats only)" -foreground brown
$w.f.opts.metaopts.analysis add command 7 -label "Full first-level analysis"
$w.f.opts.metaopts.analysis add command 1 -label "Pre-Stats"
$w.f.opts.metaopts.analysis add command 3 -label "Pre-Stats + Stats"
$w.f.opts.metaopts.analysis add command 2 -label "                     Stats"
$w.f.opts.metaopts.analysis add command 6 -label "                     Stats + Contrasts, Thresholding, Rendering"
$w.f.opts.metaopts.analysis add command 4 -label "                                  Contrasts, Thresholding, Rendering" -foreground brown
$w.bhelp bind $w.f.opts.metaopts.analysis -msg "You can run a full analysis - Pre-Stats; Stats; Contrasts (when GLM is
selected), Thresholding and Rendering - or a (sensible) subset of
these options.

If you select \"No first-level analysis\" or \"Constrasts,
Thresholding and Rendering\", you will need to select a FEAT directory
or directories instead of starting with one or more 4D images; the
stats results already produced will then be used. The options that
require selection of one or more FEAT directories are shown in brown.

Note that if you want to run only \"Constrasts, Thresholding and
Rendering\", you must select the FEAT directory (or directories)
before editing the contrasts or thresholding parameters, as these will
get initially reset on selection of the FEAT directory (or
directories)."

#}}}
#{{{ registration

label $w.f.opts.metaopts.reglabel -text "  Registration"
set fmri(reg_yn) 0
checkbutton $w.f.opts.metaopts.reg_yn -variable fmri(reg_yn) -command "feat4:updateanalysis $w 0"
$w.bhelp bind $w.f.opts.metaopts.reglabel -msg "Do you want to use FLIRT to automatically register the
FMRI data to high resolution and/or template images?"
$w.bhelp bind $w.f.opts.metaopts.reg_yn -msg "Do you want to use FLIRT to automatically register the
FMRI data to high resolution and/or template images?"

#}}}
#{{{ group stats

label $w.f.opts.metaopts.grouplabel -text " Group stats"
set fmri(group_yn) 0
checkbutton $w.f.opts.metaopts.group_yn -variable fmri(group_yn) -command "feat4:updateanalysis $w 0"
$w.bhelp bind $w.f.opts.metaopts.grouplabel -msg "Do you want to carry out group analyses?"
$w.bhelp bind $w.f.opts.metaopts.group_yn -msg "Do you want to carry out group analyses?"

#}}}

#pack $w.f.opts.metaopts.type $w.f.opts.metaopts.analysis $w.f.opts.metaopts.reglabel $w.f.opts.metaopts.reg_yn $w.f.opts.metaopts.grouplabel $w.f.opts.metaopts.group_yn -in $w.f.opts.metaopts -side left -anchor w

pack $w.f.opts.metaopts.analysis $w.f.opts.metaopts.reglabel $w.f.opts.metaopts.reg_yn $w.f.opts.metaopts.grouplabel $w.f.opts.metaopts.group_yn -in $w.f.opts.metaopts -side left -anchor w

#}}}

tixNoteBook $w.f.opts.nb
$w.f.opts.nb add misc -label "Misc"
$w.f.opts.nb add filtering -label "Pre-stats"
$w.f.opts.nb add poststats -label "Thresholding & rendering"
$w.f.opts.nb add reg -label "Registration"
$w.f.opts.nb add group -label "Group stats"

#{{{ Misc

set fmri(miscf) [ $w.f.opts.nb subwidget misc ]

#{{{ delay

set fmri(delay) 0

tixControl $w.f.delay -label " Delay before starting (hours)" \
	-variable fmri(delay) -step 1 -min 0 -selectmode immediate -padx 2

$w.bhelp bind $w.f.delay -msg "If you are using FEAT to carry out multiple analyses, you should
probably do this overnight, to reduce the load on the computer
system. You can use this delay feature to tell FEAT how long to wait
before starting the analyses - for example, so that FEAT does not
start before 7pm."

#}}}
#{{{ balloon help

frame $w.f.help

label $w.help_label -text "  Balloon help"
set fmri(help_yn) 1
checkbutton $w.help_yn -variable fmri(help_yn) -command "feat4:updatehelp $w"

pack $w.help_label $w.help_yn -in $w.f.help -side left -padx 0

$w.bhelp bind $w.help_label -msg "And don't expect this message to appear whilst you've turned this
option off!"

#}}}
#{{{ brain threshold

set fmri(brain_thresh) 10
tixControl $w.f.brain_thresh -label "  Brain/background threshold, % " \
	-variable fmri(brain_thresh) -step 1 -min 0 -max 100 -selectmode immediate

$w.bhelp bind $w.f.brain_thresh -msg "This is automatically calculated, as a % of the maximum input image
intensity. It is used in intensity normalisation and brain mask
generation."

#}}}
#{{{ desigm matrix picture options

set fmri(dmred) 1

#frame $w.f.dmred
#label $w.f.dmred_label -text "  Use red line graphs in design matrix picture"
#checkbutton $w.f.dmred_yn -variable fmri(dmred) -command "feat4:setup_dmred $w"
#pack $w.f.dmred_label $w.f.dmred_yn -in $w.f.dmred -side left

set fmri(dmgrey) 1

#frame $w.f.dmgrey
#label $w.f.dmgrey_label -text "  Use greyscale plots in design matrix picture"
#checkbutton $w.f.dmgrey_yn -variable fmri(dmgrey) -command "feat4:setup_dmgrey $w"
#pack $w.f.dmgrey_label $w.f.dmgrey_yn -in $w.f.dmgrey -side left

#}}}

#pack $w.f.delay $w.f.help $w.f.brain_thresh $w.f.dmred $w.f.dmgrey -in $fmri(miscf) -anchor w -side top
pack $w.f.delay $w.f.help $w.f.brain_thresh -in $fmri(miscf) -anchor w -side top

#}}}
#{{{ Pre-statistics processing

set fmri(filteringf) [ $w.f.opts.nb subwidget filtering ]

set fmri(filtering_yn) 1

#{{{ motion correction

frame $w.f.mc
set fmri(mcf) $w.f.mc

label $w.f.mc.label -text "Motion correction: "

global fmri(mc)
tixOptionMenu $w.f.mc.menu -label "Motion correction: " -variable fmri(mc) -command "feat4:updatemc $w"
$w.f.mc.menu add command 0 -label "None"
$w.f.mc.menu add command 1 -label "MCFLIRT"
set fmri(mc) 1

pack $w.f.mc.menu -in $fmri(mcf) -side top -padx 5 -side left

$w.bhelp bind $w.f.mc -msg "You will normally want to apply motion correction; this attempts to
remove the effect of subject head motion during the
experiment. MCFLIRT uses FLIRT (FMRIB's Linear Registration Tool)
tuned to the problem of FMRI motion correction, applying rigid-body
transformations."

#}}}
#{{{ spin history

set fmri(sh_yn) 0

#frame $w.f.sh
#
#label $w.f.sh.label -text "Adjustment for movement"
#
#checkbutton $w.f.sh.yn -variable fmri(sh_yn)
#
#pack $w.f.sh.label $w.f.sh.yn -in $w.f.sh -padx 5 -side left
#
#$w.bhelp bind $w.f.sh -msg "It is normally recommended that SPM \"adjustment for movement\" (often
#referred to as \"spin history correction\") be applied. This attempts
#to remove temporal effects which are correlated with the estimated
#motion correction parameters, as these are likely to be motion-induced
#rather than stimulation-induced. FEAT automatically sets the correct
#type of \"adjustment for movement\"."

#}}}
#{{{ BET brain extraction

frame $w.f.bet

label $w.f.bet.label -text "BET brain extraction"

set fmri(bet_yn) 1
checkbutton $w.f.bet.yn -variable fmri(bet_yn)

pack $w.f.bet.label $w.f.bet.yn -in $w.f.bet -side left -padx 3

$w.bhelp bind $w.f.bet -msg "This uses BET brain extraction to create a brain mask from the first
volume in the data. This is normally better than simple
intensity-based thresholding for getting rid of unwanted voxels in
FMRI data. Note that here, BET is used in a quite liberal way so that
there is very little danger of removing valid brain voxels."

#}}}
#{{{ spatial filtering

set fmri(smooth) 5
tixControl $w.f.smooth -label " Spatial smoothing FWHM (mm)" \
	-variable fmri(smooth) -step 1 -min 0 -selectmode immediate

$w.bhelp bind $w.f.smooth -msg "This determines the extent of the spatial smoothing, carried out on
each volume of the FMRI data set separately. This is intended to
reduce noise without reducing valid activation; this is successful as
long as the underlying activation area is larger than the extent of
the smoothing. Thus if you are looking for very small activation areas
then you should maybe reduce smoothing from the default of 5mm, and if
you are looking for larger areas, you can increase it, maybe to 10 or
even 15mm."

#}}}
#{{{ intensity normalization

frame $w.f.norm

label $w.f.norm.label -text "Intensity normalization"

set fmri(norm_yn) 0
checkbutton $w.f.norm.yn -variable fmri(norm_yn)

pack $w.f.norm.label $w.f.norm.yn -in $w.f.norm -side left -padx 3

$w.bhelp bind $w.f.norm -msg "This forces every FMRI volume to have the same mean intensity. For
each volume it calculates the mean intensity and then scales the
intensity across the whole volume so that the global mean becomes a
preset constant. This step is normally discouraged - hence is turned
off by default. When this step is not carried out, the whole 4D data
set is still normalised by a single scaling factor (\"grand mean
scaling\") - each volume is scaled by the same amount. This is so that
inter-subject and inter-session second-level analyses are valid."

#}}}
#{{{ temporal filtering

frame $w.f.temp

label $w.f.temp.label -text " Temporal filtering    "

set fmri(temphp_yn) 1
label $w.f.temp.hplabel -text "Highpass"
checkbutton $w.f.temp.hp_yn -variable fmri(temphp_yn)
set fmri(templp_yn) 0
label $w.f.temp.lplabel -text "Lowpass"
checkbutton $w.f.temp.lp_yn -variable fmri(templp_yn)

pack $w.f.temp.label $w.f.temp.hplabel $w.f.temp.hp_yn $w.f.temp.lplabel $w.f.temp.lp_yn -in $w.f.temp -side top -side left

$w.bhelp bind $w.f.temp -msg "Nonlinear \"highpass\" temporal filtering uses a local fit of a
straight line (Gaussian-weighted within the line to give a smooth
response) to remove low frequency artefacts. This is preferable to
standard linear filtering as it does not introduce new
autocorrelations into the data.

Lowpass filtering reduces high frequency noise by Gaussian smoothing,
but also can reduce the strength of the signal of interest,
particularly for single-event experiments. It is not considered to be
helpful in general, so is turned off by default. With IRVA analysis,
Gaussian smoothing lessens the validatity of the inference, so in this
case it is strongly recommended that lowpass filtering not be carried
out."

#}}}

pack $fmri(mcf) $w.f.bet $w.f.smooth $w.f.norm $w.f.temp -in $fmri(filteringf) -anchor w -pady 0

#}}}
#{{{ Post-Stats

set fmri(lfpoststats) [ $w.f.opts.nb subwidget poststats ]

set fmri(poststats_yn) 1

#{{{ thresholding

tixLabelFrame $w.f.thresh -label "Thresholding"
set fmri(lfthresh) [ $w.f.thresh subwidget frame ]

global fmri(thresh)
tixOptionMenu $w.f.thresh.menu -variable fmri(thresh)
$w.f.thresh.menu add command 0 -label "None"
$w.f.thresh.menu add command 1 -label "Uncorrected"
$w.f.thresh.menu add command 2 -label "Voxel"
$w.f.thresh.menu add command 3 -label "Cluster"

set fmri(prob_thresh) 0.01
set fmri(z_thresh) 2.3

tixControl $w.f.uncorrected -label "Uncorrected voxel P threshold" \
	-variable fmri(prob_thresh) -step 0.005 -min 0 -max 1 -selectmode immediate
tixControl $w.f.voxel -label "Corrected voxel P threshold" \
	-variable fmri(prob_thresh) -step 0.005 -min 0 -max 1 -selectmode immediate
tixControl $w.f.prob_thresh -label " Cluster P threshold" \
	-variable fmri(prob_thresh) -step 0.005 -min 0 -max 1 -selectmode immediate
tixControl $w.f.z_thresh -label "Z threshold" \
	-variable fmri(z_thresh) -step 0.1 -min 0 -selectmode immediate

pack $w.f.thresh.menu -in $fmri(lfthresh) -side top -padx 5 -side left

$w.bhelp bind $w.f.thresh -msg "After carrying out the initial statistical test, the resulting Z
statistic image is then normally thresholded to show which voxels or
clusters of voxels are activated at a particular significance level.

If \"Cluster\" thresholding is selected, a Z statistic threshold is
used to define contiguous clusters.  Then each cluster's estimated
significance level is compared with the cluster probability
threshold. Significant clusters are then used to mask the original Z
statistic image for later production of colour blobs. (This last stage
is not normally done by MEDx - MEDx normally stops at the production
of significant clusters. Note also that the Z statistic values which
survived masking by the clusters are still uncorrected Z statistics.)
This method of thresholding is an alternative to multiple comparison
correction (\"Voxel\"), and is normally more sensitive to
activation. You may well want to increase the \"Cluster creation Z
threshold\" if you have reasonable or high levels of activation.

If \"Voxel\" thresholding is selected, multiple comparison resel
correction is carried out, with thresholding at the chosen level. The
difference between this and Bonferoni multiple comparison correction
is that the number of voxels used in the calculation is reduced from
the original number of brain voxels to the number of \"independent\"
voxels, or \"resels\". Thus this test is less overly-conservative than
Bonferoni correction.

You can also choose to simply threshold the uncorrected Z statistic
values, or apply no thresholding at all."

#}}}
#{{{ rendering

tixLabelFrame $w.f.render -label "Rendering"
set fmri(lfrendering) [ $w.f.render subwidget frame ]

#{{{ Z display min and max

global fmri(zdisplay)
tixOptionMenu $w.f.zmaxmenu -variable fmri(zdisplay)
$w.f.zmaxmenu add command 0 -label "Use actual Z min/max"
$w.f.zmaxmenu add command 1 -label "Use preset Z min/max"

set fmri(zmin) 1
tixControl $w.f.zmin -label "Min" -variable fmri(zmin) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }
set fmri(zmax) 15
tixControl $w.f.zmax -label "Max" -variable fmri(zmax) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }

$w.bhelp bind $w.f.zmaxmenu -msg "The Z statistic range selected for rendering is automatically
calculated by default, to run from red (minimum Z statistic after
thresholding) to yellow (maximum Z statistic). If more than one colour
rendered image is to be produced (i.e., when multiple constrasts are
created) then the overall range of Z values is automatically found
from all of the Z statistic images, for consistent Z statistic
colour-coding.

If multiple analyses are to be carried out, \"Use preset Z min/max\"
should be chosen, and the min/max values set by hand. Again, this
ensures consistency of Z statistic colour-coding - if several
experiments are to be reported side-by-side, colours will refer to the
same Z statistic values in each picture. When using this option, you
should choose a conservatively wide range for the min and max (e.g.,
min=1, max=15), to make sure that you do not carry out unintentional
thresholding via colour rendering."

#}}}
#{{{ render type

tixOptionMenu $w.f.rendertype -variable fmri(rendertype)

$w.f.rendertype add command 0 -label "Solid blobs"
$w.f.rendertype add command 1 -label "Transparent blobs"

set fmri(rendertype) 1

$w.bhelp bind $w.f.rendertype -msg "With \"Solid colours\" you don't see any sign of the background images
within the colour blobs; with \"Transparent colours\" you will see
through the colour blobs to the background intensity"

#}}}

pack $w.f.zmaxmenu $w.f.rendertype -in $fmri(lfrendering) -side left -anchor n -padx 5 -pady 5

#}}}

pack $w.f.thresh $w.f.render -in $fmri(lfpoststats) -side top -anchor w

#}}}
#{{{ Registration

set fmri(regf) [ $w.f.opts.nb subwidget reg ]

#{{{ standard

tixLabelFrame $w.f.standard -label "Standard image"
set fmri(standardf) [ $w.f.standard subwidget frame ]

set fmri(regstandard_yn) 1
checkbutton $w.f.standard.do_yn -variable fmri(regstandard_yn) -command "set fmri(reghighres_yn) 1 ; feat4:updatereg $w"

set fmri(regstandard) ${FSLDIR}/etc/standard/avg152T1_brain.hdr

FSLFileEntry $w.f.standardimage \
	-variable fmri(regstandard) \
	-pattern "*.hdr" \
	-directory "${FSLDIR}/etc/standard/" \
	-label "" \
	-title "Select the standard image" \
	-width 50 \
	-filterhist VARS(history)

pack $w.f.standard.do_yn -in $fmri(standardf) -side left -padx 2 -pady 2

$w.bhelp bind $w.f.standard -msg "This is the standard (reference) image used in the two-stage
registration; it should be an image already in Talairach space,
ideally with the non-brain structures already removed."

$w.bhelp bind $w.f.standardimage -msg "This is the standard (reference) image used in the two-stage
registration; it should be an image already in Talairach space,
ideally with the non-brain structures already removed."

#}}}
#{{{ high res

tixLabelFrame $w.f.highres -label "Subject's high resolution image"
set fmri(highresf) [ $w.f.highres subwidget frame ]

set fmri(reghighres_yn) 1
checkbutton $w.f.highres.do_yn -variable fmri(reghighres_yn) -command "set fmri(regstandard_yn) 1 ; feat4:updatereg $w"

frame $w.f.highresselect

set highres_files(1) "~"

FSLFileEntry $w.f.highresimage \
	-variable highres_files(1) \
	-pattern "*.hdr" \
	-directory $PWD \
	-label "" \
	-title "Select the high resolution image" \
	-width 50 \
	-filterhist VARS(history)

button $w.f.highresmultiple -text "Select high res images" -command "feat4:multiple_regselect $w"

pack $w.f.highresimage -in $w.f.highresselect

pack $w.f.highres.do_yn -in $fmri(highresf) -side left -padx 2 -pady 2

$w.bhelp bind $w.f.highres -msg "This is the high resolution structural image which the low resolution
functional data will be registered to, and this in turn will be
registered to the standard brain. It is strongly recommended that this
image have non-brain structures already removed, for example by using BET."

$w.bhelp bind $w.f.highresimage -msg "This is the high resolution structural image which the low resolution
functional data will be registered to, and this in turn will be
registered to the standard brain. It is strongly recommended that this
image have non-brain structures already removed, for example by using BET."

#}}}

pack $w.f.standard $w.f.highres -in $fmri(regf) -side top -anchor w -pady 0

#}}}
#{{{ Group Stats

set fmri(lfgroup) [ $w.f.opts.nb subwidget group ]

set fmri(ngroupa) 0
tixControl $w.f.ngroupa -label "Number in group A (control group) " -padx 2 \
	-variable fmri(ngroupa) -step 1 -min 0 -max [ FSLmax 0 [ expr $fmri(multiple) - 1 ] ] \
	-selectmode immediate \
	-options { entry.width 3 }

pack $w.f.ngroupa -in $fmri(lfgroup) -side top -anchor w -padx 5

$w.bhelp bind $w.f.ngroupa -msg "You are going to carry out a multi-session second-level analysis.

This can either be a single group of sessions/subjects tested against
zero, or can be group B (test group) tested against group A (control
group).

For single-group analysis, leave the group A count as 0.

For two-group analysis, which is based on \"B-A\" tests, set the group
A count. The memberships of the groups are then set according to the
order of the entries in the multiple analysis section at the top of
the GUI, with the group A sessions first, and then the group B
sessions."

#}}}

set fmri(thresh) 3
set fmri(zdisplay) 0
$w.f.thresh.menu configure -command "feat4:updatepoststats $w"
$w.f.zmaxmenu configure -command "feat4:updatepoststats $w"
$w.f.multiple.number configure -command "feat4:updateselect $w"

pack $w.f.opts.metaopts $w.f.opts.nb -in $w.f.opts.b -side top -anchor w -pady 5 -padx 5

$w.f.opts.metaopts.analysis configure -command "feat4:updateanalysis $w"
set fmri(analysis) 7
feat4:updatereg $w

#}}}
    #{{{ Button Frame

frame $w.btns
frame $w.btns.b -relief raised -borderwidth 1
    
button $w.apply -command "feat4:apply $w" -text "Go"

FSLFileDirSelectDialog $w.savedialog \
	-title  "Save FEAT setup" \
	-command "feat4:write $w 1"
$w.savedialog subwidget fsbox config -directory $PWD -pattern "*.fsf" -filterhist VARS(history)
button $w.save -command "$w.savedialog popup" -text "Save"

FSLFileDirSelectDialog $w.loaddialog \
	-title  "Load FEAT setup" \
	-command "feat4:load $w 1"
$w.loaddialog subwidget fsbox config -directory $PWD -pattern "*.fsf" -filterhist VARS(history)
button $w.load -command "$w.loaddialog popup" -text "Load"

button $w.cancel -command "destroy $w" -text "Exit"

button $w.help -command "FmribWebHelp file: ${FSLDIR}/doc/feat4/index.html" -text "Help"

#{{{ Utils

menubutton $w.utils -text "Utils" -menu $w.utils.menu -relief raised

menu $w.utils.menu

if { $INMEDX } {
    $w.utils.menu add command -label "Load FEAT session into MEDx" -command "feat4:medxload 0"
}

if { ! $INMEDX } {
    $w.utils.menu add command -label "Stats colour rendering " -command { catch { exec sh -ce "${FSLDIR}/bin/Renderstats" & } }
}

$w.utils.menu add command -label "Colour render FEAT stats in high res" -command { catch { exec sh -ce "${FSLDIR}/bin/Renderhighres" & } }

#}}}

pack $w.btns.b -side bottom -fill x -padx 3 -pady 3
pack $w.apply $w.save $w.load $w.cancel $w.help $w.utils -in $w.btns.b \
	-side left -expand yes -padx 3 -pady 3 -fill y

pack $w.btns -in $w -side bottom -fill x

#}}}
    #{{{ Load fsf/hdr file

if { $argc > 0 && [ string length [ lindex $argv 0 ] ] > 0 } {

    set inputname [ lindex $argv 0 ]

    if { [ string first / $inputname ] != 0 && [ string first ~ $inputname ] != 0 } {
	set inputname ${PWD}/$inputname
    }

    if { [ string compare [ file extension $inputname ] .fsf ] == 0 } {

	puts "Loading Feat setup file $inputname"
	feat4:load $w 1 $inputname

    } elseif { [ string compare [ file extension $inputname ] .hdr ] == 0 } {

	puts "Selected functional data $inputname"
	set feat_files(1) $inputname
	feat4:multiple_check $w 0
	
    }
}

#}}}
}

#}}}
#{{{ feat4:apply

proc feat4:apply { w } {
    global fmri feat_files highres_files FSLDIR HOME INMEDX

    #{{{ check FILM setup

if { $fmri(paradigm_type) == 3 && $fmri(analysis) > 1 && ! $fmri(filmsetup) } {
    MxPause "Please setup model-based details"
    return 1
}

#}}}
    #{{{ update variables

foreach v { tr ndelete paradigm_hp mf_cycle brain_thresh smooth uncorrected voxel prob_thresh z_thresh zmin zmax } {
    $w.f.$v update
}

#}}}
    #{{{ if using a MEDx page, save the page

set fmri(medxload) 0

if { $fmri(multiple) < 1 } {

    set GroupFullNameTmp $fmri(groupname)

    regsub -all " " $GroupFullNameTmp "" GroupFullNameTmp2
    regsub -all "\"" $GroupFullNameTmp2 "" GroupFullName

    # find directory and filetailroot
    set GroupDirName [ file dirname  $GroupFullName ]
    regsub -all "\\." [ file rootname [ file tail $GroupFullName ] ] "_" GroupFileRoot

    set NotTmp [ string compare /tmp [ string range $GroupDirName 0 3 ] ]
    if { ! [ file writable $GroupDirName ] || $NotTmp == 0 } {
	set GroupName ${HOME}/${GroupFileRoot}
    } else {
	set GroupName ${GroupDirName}/${GroupFileRoot}
    }

    set tmpname [ new_filename ${GroupName}.hdr ]

    if { ! [ info exists feat_files(1) ] || [ FSLSaveAs $feat_files(1) AVW $tmpname true ] } {
	MxPause "Please select a valid group page"
	return 1
    }

    set feat_files(1) $tmpname
    
    set fmri(multiple) 1
    set fmri(medxload) 1
}

#}}}
    #{{{ write model

if { $fmri(analysis)>0 } {
    set createthemodel 1
} else {
    set createthemodel 0
}

if { [ feat4:write $w $createthemodel $fmri(feat_filename) ] } {
    return 1
}

#}}}
    #{{{ if group_yn test for existing registration

if { $fmri(group_yn) && !$fmri(reg_yn) && ( $fmri(analysis) == 0 || $fmri(analysis) == 4 ) } {

    set problem 0

    set i 1
    while { $i <= $fmri(multiple) } {
	if { ! [ file exists $feat_files($i)/example_func2standard.mat ] } {
	    set problem 1
	}
	incr i 1
    }

    if { $problem } {
	MxPause "Registration has not been run for all of the FEAT directories that you have selected for group analysis.

Please turn on and setup registration."
	return 1
    }

}

#}}}
    #{{{ wait if required

set thedelay ""
if { $fmri(delay) > 0.003 } {
    set thedelay "sleep [ expr int( $fmri(delay) * 60.0 * 60.0 ) ] ; "
}

#}}}

    set FSFROOT [ file rootname $fmri(feat_filename) ]

    if { ! $INMEDX } {
	catch { exec sh -c "$thedelay ${FSLDIR}/bin/feat $FSFROOT" & } junk
	MxPause "Running FEAT using setup file
${FSFROOT}.fsf"
    } else {
	feat4:proc $FSFROOT
	if { $fmri(medxload) } {
	    feat4:medxload_proc $feat_files(1) 1
	}
    }

    update idletasks
}

#}}}
#{{{ updates

#{{{ feat4:updatetype

proc feat4:updatetype { w force dummy } {
    global fmri

    if { $force } {
	if { $fmri(type) == 1 } {
	    set fmri(mc) 0
	} elseif { $fmri(type) == 2 } {
	    set fmri(mc) 1
	}
    }
}

#}}}
#{{{ feat4:updateparadigm

proc feat4:updateparadigm { w process force dummy } {
    global fmri

    if { $process } {
	set fmri(paradigm_type) 3
	if { [ info exists fmri(w_model) ] && [ winfo exists $fmri(w_model) ] } {
	    destroy $fmri(w_model)
	}
	feat4:setup_model_vars_simple $w
	if { [ feat4:setup_model_preview $w ] } {
	    return 1
	}
    }

    if { $force } {
	set fmri(templp_yn) 0
    }

    if { $fmri(paradigm_type) == 3 } {
	pack forget $w.f.mf
	pack $w.f.ud -in $w.f.paradigmmain -side left
    } else {
	set fmri(a_count) 20
	set fmri(b_count) 0
	pack forget $w.f.ud
	pack $w.f.mf -in $w.f.paradigmmain -side left
	feat4:updategroup $w
    }
}

#}}}
#{{{ feat4:updatemf

proc feat4:updatemf { w dummy } {
    global fmri

    if { $fmri(mf_type) == 1 } {
	pack forget $w.f.mf_vols
	pack forget $w.f.mf_file
	pack $w.f.mf_cycle -in $w.f.mf -side left
    } else {
	pack forget $w.f.mf_cycle
	pack $w.f.mf_vols $w.f.mf_file -in $w.f.mf -side left
    }

    set fmri(paradigm_hp) [ expr $fmri(mf_cycle) * $fmri(tr) * 1.5 ]
}

#}}}
#{{{ feat4:updateanalysis

proc feat4:updateanalysis { w dummy } {
    global fmri

    set fmri(filtering_yn) [ expr   $fmri(analysis)       % 2 == 1 ]
    set fmri(stats_yn)     [ expr ( $fmri(analysis) / 2 ) % 2 == 1 ]
    set fmri(poststats_yn) [ expr ( $fmri(analysis) / 4 ) % 2 == 1 ]

    feat4:updategroup $w
    feat4:updatereg $w

    if { $fmri(group_yn) } {
	set fmri(poststats_yn) 1
    }

    #{{{ update notebook view

if { !$fmri(filtering_yn) } {
    $w.f.opts.nb pageconfigure filtering -state disabled
}  else {
    $w.f.opts.nb pageconfigure filtering -state normal
}

if { !$fmri(poststats_yn) } {
    $w.f.opts.nb pageconfigure poststats -state disabled
} else {
    $w.f.opts.nb pageconfigure poststats -state normal
    if { !$fmri(stats_yn) } {
	set fmri(ndelete) 0
    }
}

#}}}
    #{{{ update main section view

if { $fmri(analysis) == 0 } {

    pack forget $w.f.datamain
    pack forget $w.f.paradigmmain
    pack forget $w.f.paradigmmainalt

} elseif { $fmri(analysis) == 4 } {

    pack forget $w.f.datamain
    pack forget $w.f.paradigmmain
    if { $fmri(paradigm_type) != 4 } {
	pack $w.f.paradigmmainalt -in $fmri(lfparadigm)
    }

} else {

    pack $w.f.datamain -in $fmri(lfdata)
    pack $w.f.paradigmmain -in $fmri(lfparadigm)
    pack forget $w.f.paradigmmainalt

}

#}}}

    feat4:updatepoststats $w 0

    $w.f.opts.nb raise misc
}

#}}}
#{{{ feat4:updateselect

proc feat4:updateselect { w dummy } {
    global fmri INMEDX feat_files

    if { $fmri(group_yn) } {
	$w.f.multiple.number configure -min 2
	if { $fmri(multiple) < 2 } {
	    set fmri(multiple) 2
	} 
    } else {
	if { $INMEDX } {
	    $w.f.multiple.number configure -min 0
	} else {
	    $w.f.multiple.number configure -min 1
	}
    }

    if { $fmri(analysis) != 4 && $fmri(analysis) != 0 } {
	if { $fmri(multiple) < 1 } {
	    $w.f.multiple.setup configure -text "Select group"
	    if { [ info exists fmri(groupname) ] } {	    
		MxGetCurrentFolder Folder
		MxGetPageByName $Folder $fmri(groupname) fmri(grouppage)
		MxGetPagesFromGroup $fmri(grouppage) GPages
		if { [ info exists GPages ] } {
		    $w.f.groupname configure -text $fmri(groupname)
		    pack $w.f.groupname -in $w.f -after $w.f.multiple -anchor e -padx 10
		    set fmri(npts) [ llength $GPages ]
		    set feat_files(1) $fmri(grouppage)
		    feat4:updatenpts $w 1
		} else {
		    set feat_files(1) ""
		}
	    }
	} else {
	    pack forget $w.f.groupname
	    $w.f.multiple.setup configure -text "Select 4D data"
	}
    } else {
	if { $fmri(multiple) == 1 } {
	    $w.f.multiple.setup configure -text "Select FEAT directory"
	} else {
	    $w.f.multiple.setup configure -text "Select FEAT directories"
	}
    }

    if { $fmri(multiple) < 2 } {
	pack forget $w.f.highresmultiple
	pack $w.f.highresimage -in $w.f.highresselect
    } else {
	pack forget $w.f.highresimage
	pack $w.f.highresmultiple -in $w.f.highresselect
    }

    $w.f.ngroupa configure -max [ FSLmax 0 [ expr $fmri(multiple) - 1 ] ]
}

#}}}
#{{{ feat4:updatehelp

proc feat4:updatehelp { w } {
    global fmri

    if { $fmri(help_yn) == 1 } {
	$w.bhelp configure -state balloon
    } else {
	$w.bhelp configure -state none
    }
}

#}}}
#{{{ feat4:updatemc

proc feat4:updatemc { w dummy } {
    global fmri

#    if { $fmri(mc) != 4 } {
#	pack forget $w.f.sh
#    } else {
#	pack $w.f.sh -in $fmri(mcf) -side left
#    }
}

#}}}
#{{{ feat4:updatepoststats

proc feat4:updatepoststats { w dummy } {
    global fmri

    if { !$fmri(thresh) } {
	pack forget $w.f.uncorrected ; pack forget $w.f.voxel ; pack forget $w.f.z_thresh ; pack forget $w.f.prob_thresh
	pack forget $w.f.render
    } else {
	pack $w.f.render -in $fmri(lfpoststats) -after $w.f.thresh -side top -anchor w -pady 5

	if { $fmri(thresh) == 1 } {
	    pack forget $w.f.voxel ; pack forget $w.f.z_thresh ; pack forget $w.f.prob_thresh
	    pack $w.f.uncorrected -in $fmri(lfthresh) -side left
	} elseif { $fmri(thresh) == 2 } {
	    pack forget $w.f.uncorrected ; pack forget $w.f.z_thresh ; pack forget $w.f.prob_thresh
	    pack $w.f.voxel -in $fmri(lfthresh) -side left
	} else {
	    pack forget $w.f.uncorrected ; pack forget $w.f.voxel
	    pack $w.f.z_thresh $w.f.prob_thresh -in $fmri(lfthresh) -side left -anchor w -pady 2
#	    set fmri(z_thresh) 2.3 ;
	}
    }

    if { !$fmri(zdisplay) } {
	pack forget $w.f.zmin
	pack forget $w.f.zmax
    } else {
	pack $w.f.zmin $w.f.zmax -in $fmri(lfrendering) -after $w.f.zmaxmenu -side left -anchor w -padx 3 -pady 5
    }
}

#}}}
#{{{ feat4:updatereg

proc feat4:updatereg { w } {
    global fmri

    if { !$fmri(reg_yn) } {
	$w.f.opts.nb pageconfigure reg -state disabled
	$w.f.opts.nb raise misc

	if { $fmri(analysis) == 0 } {
	    set fmri(group_yn) 1
	    feat4:updategroup $w
	}
    } else {
	$w.f.opts.nb pageconfigure reg -state normal

	if { $fmri(regstandard_yn) } {
	    pack $w.f.standardimage -in $fmri(standardf) -after $w.f.standard.do_yn -side left -padx 2 -pady 2
	} else {
	    pack forget $w.f.standardimage
	}

	if { $fmri(reghighres_yn) } {
	    pack $w.f.highresselect -in $fmri(highresf) -after $w.f.highres.do_yn -side left -padx 2 -pady 2
	} else {
	    pack forget $w.f.highresselect
	}

	$w.f.opts.nb raise reg
    }
}

#}}}
#{{{ feat4:updategroup

proc feat4:updategroup { w } {
    global fmri

    if { $fmri(analysis)==1 } {
	set fmri(group_yn) 0
    }

    if { !$fmri(group_yn) } {
	$w.f.opts.nb pageconfigure group -state disabled
	$w.f.opts.nb raise misc

	if { $fmri(analysis) == 0 } {
	    set fmri(reg_yn) 1
	    feat4:updatereg $w
	}
    } else {
	$w.f.opts.nb pageconfigure group -state normal
	$w.f.opts.nb raise group

#	if { $fmri(analysis) != 0 } {
#	    set fmri(reg_yn) 1
#	    feat4:updatereg $w
#	}
    }

    feat4:updateselect $w 0
}

#}}}
#{{{ feat4:updatenpts

proc feat4:updatenpts { w i } {
    global FSLDIR feat_files fmri

    set npts [ exec ${FSLDIR}/bin/avwnvols [ file rootname $feat_files($i) ] 2> /dev/null ]

    if { $npts > 0 } {
	set fmri(npts) $npts
    }

    $w.f.npts configure -text "  Total volumes $fmri(npts)"

    $w.f.mf_cycle configure -max [ expr ( $fmri(npts) - $fmri(ndelete) ) / 2 ]
}

#}}}
#{{{ feat4:setup_dmred,dmgrey

proc feat4:setup_dmred { w } {
    global fmri

    if { $fmri(dmred) == 0 } {
	set fmri(dmgrey) 1
    }
}

proc feat4:setup_dmgrey { w } {
    global fmri

    if { $fmri(dmgrey) == 0 } {
	set fmri(dmred) 1
    }
}

#}}}

#}}}
#{{{ feat4:write

proc feat4:write { w feat_model filename } {
    global fmri FSLDIR feat_files highres_files

    set filename [ file rootname $filename ].fsf

    set channel [ open $filename "w" ]

    puts $channel "
# FEAT version number
set fmri(version) $fmri(version)"

    puts $channel "
# Obsolete option
set fmri(type) $fmri(type)"

    puts $channel "
# Which stages to run
# 0 : No first-level analysis (registration and/or group stats only)
# 7 : Full first-level analysis
# 1 : Pre-Stats
# 3 : Pre-Stats + Stats
# 2 :             Stats
# 6 :             Stats + Contrasts, Thresholding, Rendering
# 4 :                     Contrasts, Thresholding, Rendering
set fmri(analysis) $fmri(analysis)"

    puts $channel "
# Analysis type
# 3 : FILM model-based
# 4 : IRVA semi-model-free
set fmri(paradigm_type) $fmri(paradigm_type)"

    puts $channel "
# TR(s)
set fmri(tr) $fmri(tr)"

    puts $channel "
# Total volumes
set fmri(npts) $fmri(npts)"

    puts $channel "
# Delete volumes
set fmri(ndelete) $fmri(ndelete)"

    puts $channel "
# Number of first-level analyses
set fmri(multiple) $fmri(multiple)"

    puts $channel "
# Delay before starting (hours)
set fmri(delay) $fmri(delay)"

    puts $channel "
# Balloon help
set fmri(help_yn) $fmri(help_yn)"

    puts $channel "
# Carry out pre-stats processing?
set fmri(filtering_yn) $fmri(filtering_yn)"

    puts $channel "
# Brain/background threshold, %
set fmri(brain_thresh) $fmri(brain_thresh)"

    puts $channel "
# Motion correction
# 0 : None
# 1 : MCFLIRT
set fmri(mc) $fmri(mc)"

    puts $channel "
# Spin-history (obsolete)
set fmri(sh_yn) $fmri(sh_yn)"

    puts $channel "
# BET brain extraction
set fmri(bet_yn) $fmri(bet_yn)"

    puts $channel "
# Spatial smoothing FWHM (mm)
set fmri(smooth) $fmri(smooth)"

    puts $channel "
# Intensity normalization
set fmri(norm_yn) $fmri(norm_yn)"

    puts $channel "
# Highpass temporal filtering
set fmri(temphp_yn) $fmri(temphp_yn)"

    puts $channel "
# Lowpass temporal filtering
set fmri(templp_yn) $fmri(templp_yn)"

    puts $channel "
# Carry out main stats?
set fmri(stats_yn) $fmri(stats_yn)"

    puts $channel "
# Number of EVs
set fmri(evs) $fmri(evs)"

    puts $channel "
# Number of contrasts
set fmri(ncon) $fmri(ncon)"

    puts $channel "
# Number of F-tests
set fmri(nftests) $fmri(nftests)"

    puts $channel "
# Use red line graphs in design matrix picture
set fmri(dmred) $fmri(dmred)"

    puts $channel "
# Use greyscale plots in design matrix picture
set fmri(dmgrey) $fmri(dmgrey)"

    puts $channel "
# Add constant column to design matrix? (obsolete)
set fmri(constcol) $fmri(constcol)"

    puts $channel "
# IRVA mode
# 1 : Cyclic
# 2 : Triggered
set fmri(mf_type) $fmri(mf_type)"

    puts $channel "
# IRVA : Volumes for complete cycle / Volumes per event
set fmri(mf_cycle) $fmri(mf_cycle)"

    puts $channel "
# IRVA : File of events
set fmri(mf_file) $fmri(mf_file)"

    puts $channel "
# Carry out post-stats steps?
set fmri(poststats_yn) $fmri(poststats_yn)"

    puts $channel "
# Thresholding
# 0 : None
# 1 : Uncorrected
# 2 : Voxel
# 3 : Cluster
set fmri(thresh) $fmri(thresh)"

    puts $channel "
# P threshold
set fmri(prob_thresh) $fmri(prob_thresh)"

    puts $channel "
# Z threshold
set fmri(z_thresh) $fmri(z_thresh)"

    puts $channel "
# Z min/max for colour rendering
# 0 : Use actual Z min/max
# 1 : Use preset Z min/max
set fmri(zdisplay) $fmri(zdisplay)"

    puts $channel "
# Z min in colour rendering
set fmri(zmin) $fmri(zmin)"

    puts $channel "
# Z max in colour rendering
set fmri(zmax) $fmri(zmax)"

    puts $channel "
# Colour rendering type
# 0 : Solid blobs
# 1 : Transparent blobs
set fmri(rendertype) $fmri(rendertype)"

    puts $channel "
# Registration?
set fmri(reg_yn) $fmri(reg_yn)"

    puts $channel "
# Registration to standard image?
set fmri(regstandard_yn) $fmri(regstandard_yn)"

    puts $channel "
# Standard image
set fmri(regstandard) $fmri(regstandard)"

    puts $channel "
# Registration to subject's high resolution image?
set fmri(reghighres_yn) $fmri(reghighres_yn)"

    puts $channel "
# Group stats
set fmri(group_yn) $fmri(group_yn)"

    puts $channel "
# Number in group A (control group)
set fmri(ngroupa) $fmri(ngroupa)"

    puts $channel "
# High pass filter cutoff
set fmri(paradigm_hp) $fmri(paradigm_hp)"

    if { $fmri(multiple) > 0 } {
	if { $w != -1 } {
	    if { [ feat4:multiple_check $w 0 ] } {
		return 1
	    }
	}
	set i 1
	while { $i <= $fmri(multiple) } {
	    puts $channel "
# 4D AVW data or FEAT directory ($i)
set feat_files($i) $feat_files($i)"
	    incr i 1
	}
    }

    if { $fmri(reg_yn) && $fmri(reghighres_yn) } {
	if { $w != -1 } {
	    if { [ feat4:multiple_regcheck $w 0 ] } {
		return 1
	    }
	}
	if { $fmri(multiple) < 2 } {
	    set nhighres 1
	} else {
	    set nhighres $fmri(multiple)
	}
	set i 1
	while { $i <= $nhighres } {
	    puts $channel "
# High resolution image for subject $i
set highres_files($i) $highres_files($i)"
	    incr i 1
	}
    }

    if { $fmri(paradigm_type)==3 } {
	set i 1
	while { $i <= $fmri(evs) } {

	    if { [ info exists fmri(shape$i) ] } {

		puts $channel "
# Basic waveform shape (EV $i)
# 0 : Square
# 1 : Sinusoid
# 2 : Custom (1 entry per volume)
# 3 : Custom (3 column format)
set fmri(shape$i) $fmri(shape$i)"

		puts $channel "
# Convolution (EV $i)
# 0 : None
# 1 : Gaussian
# 2 : Gamma
# 3 : SPM99 HRF
set fmri(convolve$i) $fmri(convolve$i)"

		puts $channel "
# Apply temporal filtering (EV $i)
set fmri(tempfilt_yn$i) $fmri(tempfilt_yn$i)"

		puts $channel "
# Add temporal derivative (EV $i)
set fmri(deriv_yn$i) $fmri(deriv_yn$i)"

		switch $fmri(shape$i) {
		    0 { 
			puts $channel "
# Skip (EV $i)
set fmri(skip$i) $fmri(skip$i)"

			puts $channel "
# Off (EV $i)
set fmri(off$i) $fmri(off$i)"

			puts $channel "
# On (EV $i)
set fmri(on$i) $fmri(on$i)"

			puts $channel "
# Phase (EV $i)
set fmri(phase$i) $fmri(phase$i)"

			puts $channel "
# Stop (EV $i)
set fmri(stop$i) $fmri(stop$i)"
		    }
		    1 { 
			puts $channel "
# Skip (EV $i)
set fmri(skip$i) $fmri(skip$i)"

			puts $channel "
# Period (EV $i)
set fmri(period$i) $fmri(period$i)"

			puts $channel "
# Phase (EV $i)
set fmri(phase$i) $fmri(phase$i)"

			puts $channel "
# Stop (EV $i)
set fmri(stop$i) $fmri(stop$i)"
		    }
		    2 { puts $channel "
# Custom EV file (EV $i)
set fmri(custom$i) $fmri(custom$i)" }
		    3 { puts $channel "
# Custom EV file (EV $i)
set fmri(custom$i) $fmri(custom$i)" }
		    4 {
			for { set j 1 } { $j < $i } { incr j 1 } {
			    puts $channel "
# Interactions (EV $i with EV $j)
set fmri(interactions${i}.$j) $fmri(interactions${i}.$j)"
			}
		    }
		}
		
		switch $fmri(convolve$i) {
		    0 { }
		    1 { 
			puts $channel "
# Gauss sigma (EV $i)
set fmri(gausssigma$i) $fmri(gausssigma$i)"

			puts $channel "
# Gauss delay (EV $i)
set fmri(gaussdelay$i) $fmri(gaussdelay$i)"
		    }
		    2 {
			puts $channel "
# Gamma sigma (EV $i)
set fmri(gammasigma$i) $fmri(gammasigma$i)"
			puts $channel "
# Gamma delay (EV $i)
set fmri(gammadelay$i) $fmri(gammadelay$i)"
		    }
		    3 { }
		}
		
		for { set j 1 } { $j < $i } { incr j 1 } {
		    puts $channel "
# Orthogonalise EV $i wrt EV $j
set fmri(ortho${i}.$j) $fmri(ortho${i}.$j)"
		}
		
	    }
	    incr i 1
	}
	
	set i 1
	while { $i <= $fmri(ncon) } {
	    puts $channel "
# Display images for contrast $i
set fmri(conpic.$i) $fmri(conpic.$i)"
	    set j 1
	    while { $j <= $fmri(evs) } {
		puts $channel "
# Contrast vector $i element $j
set fmri(con${i}.${j}) $fmri(con${i}.${j})"
		incr j 1
	    }
	    incr i 1
	}

	set i 1
	while { $i <= $fmri(nftests) } {
	    set j 1
	    while { $j <= $fmri(ncon) } {
		puts $channel "
# F-test $i element $j
set fmri(ftest${i}.${j}) $fmri(ftest${i}.${j})"
		incr j 1
	    }
	    incr i 1
	}
    } elseif { $fmri(paradigm_type)==4 } {
	puts $channel "
# Display images for contrast $i
set fmri(conpic.1) 1"
    }

    close $channel
    
    if { $w != -1 } {
	set result 0
	if { $feat_model && $fmri(paradigm_type)==3 } {

	    set result [ catch { eval exec ${FSLDIR}/bin/feat_model [ file rootname $filename ] } ErrMsg ]
	    if {$result != 0} {
		MxPause "Problem with processing the model: $ErrMsg"
	    } else {
		if { [ string length $ErrMsg ] > 0 } {
		    MxPause "Problem with processing the model: $ErrMsg"
		}
	    }
	    catch { exec sh -c ${FSLDIR}/bin/convert [ file rootname $filename ].ppm [ file rootname $filename ].gif } ErrMsg
	    catch { exec sh -c ${FSLDIR}/bin/convert [ file rootname $filename ]_cov.ppm [ file rootname $filename ]_cov.gif } ErrMsg
	}

	return $result
    }
}

#}}}
#{{{ feat4:load

proc feat4:load { w full filename } {
    global fmri feat_files highres_files VERSION

    #{{{ work out fsf version

set channel [ open $filename "r" ]

gets $channel line

if { [ scan $line "%f" version ] < 1 } {
    set version 0
}

close $channel

#}}}

    if { $version < 1 || $version > 4 } {

	if { $w != -1 } {
	    set analysis $fmri(analysis)
	    set multiple $fmri(multiple)
	    set group_yn $fmri(group_yn)
	    set reg_yn   $fmri(reg_yn)
	    for { set i 1 } { $i <= $multiple } { incr i 1 } {
		if { [ info exists feat_files($i) ] } {
		    set lfeat_files($i) $feat_files($i)
		}
	    }
	}

	source ${filename}

	if { $w != -1 && $fmri(analysis) != 4 && $fmri(analysis) != 0 } {
	    feat4:updatenpts $w 1
	}

	if { ! $full } {
	    set fmri(npts) [ expr $fmri(npts) - $fmri(ndelete) ]
	    set fmri(ndelete) 0
	    set fmri(group_yn) $group_yn
	    set fmri(reg_yn)   $reg_yn
	    set fmri(analysis) $analysis
	    set fmri(multiple) $multiple
	    for { set i 1 } { $i <= $multiple } { incr i 1 } {
		set feat_files($i) $lfeat_files($i)
	    }
	}

	if { $w != -1 } {
	    feat4:updatereg $w
	    feat4:updatetype $w 0 0
	    feat4:updateparadigm $w 0 0 -1
	    if { $fmri(paradigm_hp) == 4 } {
		feat4:updatemf $w 0
	    }
	    feat4:updateanalysis $w 0
	    feat4:updateselect $w 0
	    feat4:updatehelp $w
	    feat4:updatemc $w 0
	}

    } else {

	#{{{ read old format file

set channel [ open $filename "r" ]

gets $channel line
scan $line "%f %d %d %d" version fmri(type) analysis fmri(paradigm_type)
gets $channel line
scan $line "%f %d %d" fmri(tr) fmri(npts) fmri(ndelete)
gets $channel line
scan $line "%d %f %d %d %d" multiple fmri(delay) junk junk fmri(help_yn)
gets $channel line
scan $line "%d %f %d %d %f %d %d %d %f" filtering_yn fmri(brain_thresh) fmri(mc) fmri(sh_yn) fmri(smooth) fmri(norm_yn) fmri(temphp_yn) fmri(templp_yn) fmri(paradigm_hp)
gets $channel line
scan $line "%d" stats_yn
gets $channel line
scan $line "%d %d %d %d %d" fmri(evs) fmri(ncon) fmri(dmred) fmri(dmgrey) fmri(constcol)
gets $channel line
scan $line "%d %f %s" fmri(mf_type) fmri(mf_cycle) fmri(mf_file)
gets $channel line
scan $line "%d %d %f %f" poststats_yn fmri(thresh) fmri(prob_thresh) fmri(z_thresh)
gets $channel line
scan $line "%d %f %f %d" fmri(zdisplay) fmri(zmin) fmri(zmax) fmri(rendertype)

if { $version > 3.125 } {
    gets $channel line
    scan $line "%d %d %s %d" fmri(reg_yn) fmri(regstandard_yn) fmri(regstandard) fmri(reghighres_yn)
    gets $channel line
    scan $line "%d %d" fmri(group_yn) fmri(ngroupa)
} else {
    set fmri(reg_yn) 0
    set fmri(ngroupa) 0
}

set i 1
while { $i <= $multiple } {
    gets $channel line
    if { $full } {
	scan $line "%s" feat_files($i)
    }
    incr i 1
}

if { $version > 3.125 && $fmri(reg_yn) && $fmri(reghighres_yn) } {
    if { $multiple < 2 } {
	set nhighres 1
    } else {
	set nhighres $multiple
    }
    set i 1
    while { $i <= $nhighres } {
	gets $channel line
	if { $full } {
	    scan $line "%s" highres_files($i)
	}
	incr i 1
    }
}

if { $full } {
    set fmri(analysis) $analysis
    set fmri(multiple) $multiple
} else {
    set fmri(npts) [ expr $fmri(npts) - $fmri(ndelete) ]
    set fmri(ndelete) 0
    set fmri(reg_yn) 0
}

if { $w != -1 } {
    feat4:updatereg $w
    feat4:updatetype $w 0 0
    feat4:updateparadigm $w 0 0 -1
    feat4:updatemf $w 0
    feat4:updateanalysis $w 0
    feat4:updateselect $w 0
    feat4:updatehelp $w
    feat4:updatemc $w 0
} else {
    set fmri(filtering_yn) $filtering_yn
    set fmri(stats_yn) $stats_yn
    set fmri(poststats_yn) $poststats_yn
}

if { $fmri(paradigm_type)==3 } {
	    
    if { $analysis!=0 && $w != -1 } {
	feat4:setup_model_init_evs $w
    }
	    
    set i 1
    while { $i <= $fmri(evs) } {
	gets $channel line
	scan $line "%d %d %d %d" fmri(shape$i) fmri(convolve$i) fmri(tempfilt_yn$i) fmri(deriv_yn$i)
		
	gets $channel line
	switch $fmri(shape$i) {
	    0 { scan $line "%f %f %f %f %f" fmri(skip$i) fmri(off$i) fmri(on$i) fmri(phase$i) fmri(stop$i) }
	    1 { scan $line "%f %f %f %f" fmri(skip$i) fmri(period$i) fmri(phase$i) fmri(stop$i) }
	    2 { scan $line "%s" fmri(custom$i) }
	    3 { scan $line "%s" fmri(custom$i) }
	    4 {
		for { set j 1 } { $j < $i } { incr j 1 } {
		    scan $line "%d" fmri(interactions${i}.$j)
		    if { $j < [ expr $i - 1 ] } {
			gets $channel line
		    }
		}
	    }
	}
		
	if { $fmri(convolve$i) != 0 && $fmri(convolve$i) != 3 } {
	    gets $channel line
	    switch $fmri(convolve$i) {
		1 { scan $line "%f %f" fmri(gausssigma$i) fmri(gaussdelay$i) }
		2 { scan $line "%f %f" fmri(gammasigma$i) fmri(gammadelay$i) }
	    }
	}
	    
	if { $version > 3.135 } {
	    for { set j 1 } { $j < $i } { incr j 1 } {
		gets $channel line
		scan $line "%d" fmri(ortho${i}.$j)
	    }
	} else {
	    for { set j 1 } { $j < $i } { incr j 1 } {
		set fmri(ortho${i}.$j) 0
	    }
	}
	
	incr i 1
    }
	    
    set i 1
    while { $i <= $fmri(ncon) } {
	set j 1
	while { $j <= $fmri(evs) } {
	    gets $channel line
	    scan $line "%f" fmri(con${i}.${j})
	    incr j 1
	}
	incr i 1
    }
}
    
close $channel

#}}}

    }
    
    set fmri(version) $VERSION
}

#}}}
#{{{ feat4:film_wizard

proc feat4:film_wizard { w } {
    global FSLDIR fmri

    #{{{ setup window

    set count 0
    set w0 ".dialog[incr count]"
    while { [ winfo exists $w0 ] } {
        set w0 ".dialog[incr count]"
    }

    toplevel $w0

    wm title $w0 "FILM simple model setup"
    wm iconname $w0 "FILM"
    wm iconbitmap $w0 @${FSLDIR}/tcl/fmrib.xbm

    frame $w0.f

#}}}
    #{{{ choose paradigm type

frame $w0.f.paradigm_type

set fmri(wizard_type) 1	

radiobutton $w0.f.paradigm_type.jab -text "ABAB..." -value 1 -variable fmri(wizard_type) -command "feat4:update_film_wizard $w0"
radiobutton $w0.f.paradigm_type.jabac -text "ABACABAC..." -value 2 -variable fmri(wizard_type) -command "feat4:update_film_wizard $w0"

$w.bhelp bind $w0.f.paradigm_type -msg "Choose whether to setup ABAB... or ABACABAC... designs (block or
single-event). The A blocks will normally be rest (or control)
conditions."

pack $w0.f.paradigm_type.jab $w0.f.paradigm_type.jabac -in $w0.f.paradigm_type -side top -padx 5 -side left

#}}}
    #{{{ frame ab

frame $w0.f.ab

set fmri(a_count) 30
tixControl $w0.f.a_count -label "A (rest) period (s)" \
	-variable fmri(a_count) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }

set fmri(b_count) 30
tixControl $w0.f.b_count -label "B period (s)" \
	-variable fmri(b_count) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }

set fmri(c_count) 30
tixControl $w0.f.c_count -label "C period (s)" \
	-variable fmri(c_count) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }

pack $w0.f.a_count $w0.f.b_count -in $w0.f.ab -side left -padx 5 -pady 3 -anchor n

$w.bhelp bind $w0.f.a_count -msg "The A period (seconds) is normally the rest period, i.e., no
stimulation was applied during this period."

$w.bhelp bind $w0.f.b_count -msg "The B period (seconds) is normally the activation period, i.e.,
stimulation was applied during this period."

$w.bhelp bind $w0.f.c_count -msg "The C period (seconds) is normally the period associated with a second
type of activation, i.e., stimulation type 2 was applied during this
period."

#}}}
    #{{{ setup buttons

frame $w0.btns
frame $w0.btns.b -relief raised -borderwidth 1

button $w0.cancel -command "feat4:updateparadigm $w 1 0 -1 ; destroy $w0" -text "Process"

pack $w0.btns.b -side bottom -fill x -padx 3 -pady 5
pack $w0.cancel -in $w0.btns.b -side left -expand yes -padx 3 -pady 3 -fill y

#}}}
 
    pack $w0.f.paradigm_type $w0.f.ab -in $w0.f
    pack $w0.f $w0.btns -expand yes -fill both
}

#{{{ feat4:update_film_wizard

proc feat4:update_film_wizard { w } {
    global fmri

    if { $fmri(wizard_type) == 1 } {
	set fmri(a_count) 30
	set fmri(b_count) 30
	pack forget $w.f.c_count
    } else {
	set fmri(a_count) 30
	set fmri(b_count) 30
	set fmri(c_count) 30
	pack $w.f.c_count -in $w.f.ab -after $w.f.b_count -side left -padx 5 -pady 3 -anchor n
    }
}

#}}}

#}}}
#{{{ feat4:multiple_select

proc feat4:multiple_select { w } {
    global FSLDIR feat_files fmri VARS PWD

    if { $fmri(multiple) < 1 } {
	SelectSinglePage:Dialog $w fmri(groupname) "feat4:updateselect $w 0"	 
    } else {
	#{{{ setup window

    set count 0
    set w0 ".dialog[incr count]"
    while { [ winfo exists $w0 ] } {
        set w0 ".dialog[incr count]"
    }

    toplevel $w0

    wm iconname $w0 "Select"
    wm iconbitmap $w0 @${FSLDIR}/tcl/fmrib.xbm

    frame $w0.f

    if { $fmri(analysis) != 4 && $fmri(analysis) != 0 } {
	wm title $w0 "Select 4D data"
    } else {
	if { $fmri(multiple) == 1 } {
	    wm title $w0 "Select FEAT directory"
	} else {
	    wm title $w0 "Select FEAT directories"
	}
    }

#}}}
	#{{{ setup file selections

    set i 1
    while { $i <= $fmri(multiple) } {

	if { $fmri(analysis) != 4 && $fmri(analysis) != 0 } {
	    FSLFileEntry $w0.filename$i \
		    -variable feat_files($i) \
		    -label " $i:   " \
		    -pattern "*.hdr" \
		    -directory $PWD \
		    -title "Select input data" \
		    -width 60 \
		    -filterhist VARS(history)
	} else {
	    FSLFileEntry $w0.filename$i \
		    -variable feat_files($i) \
		    -label " $i:   " \
		    -pattern "*.feat" \
		    -directory $PWD \
		    -title "Select input data" \
		    -width 60 \
		    -dirasfile "design.fsf" \
		    -filterhist VARS(history)
	}

	pack $w0.filename$i -in $w0.f -side top -expand yes -fill both -padx 3 -pady 3
	incr i 1
    }

#}}}
	#{{{ setup buttons

frame $w0.btns
frame $w0.btns.b -relief raised -borderwidth 1

button $w0.cancel -command "feat4:multiple_check $w 1; destroy $w0" -text "OK"

pack $w0.btns.b -side bottom -fill x -padx 3 -pady 5
pack $w0.cancel -in $w0.btns.b -side left -expand yes -padx 3 -pady 3 -fill y

#}}}
	pack $w0.f $w0.btns -expand yes -fill both
    }
}

proc feat4:multiple_check { w load } {
    global FSLDIR feat_files fmri

    set AllOK 1
    set i 1
    while { $i <= $fmri(multiple) } {
	if { ! [ info exists feat_files($i) ] || ! [ file exists $feat_files($i) ] } {
	    set AllOK 0
	} else {
	    if { $fmri(analysis) != 4 && $fmri(analysis) != 0 } {
		feat4:updatenpts $w $i
	    } else {
		if { ! [ file exists $feat_files($i)/design.fsf ] } {
		    set AllOK 0
		} else {
		    if { $load && $i == 1 } {
			feat4:load $w 0 $feat_files(1)/design.fsf
		    }
		}
	    }
	}
	incr i 1
    }
    
    if { ! $AllOK} {
	MxPause "Warning: you haven't filled in all the relevant selections with valid
filenames - this could include image data or FEAT directories, and
also high-resolution images."
	return 1
    }

    return 0
}

#}}}
#{{{ feat4:multiple_regselect

proc feat4:multiple_regselect { w } {
    global FSLDIR highres_files fmri VARS PWD

    #{{{ setup window

    set count 0
    set w0 ".dialog[incr count]"
    while { [ winfo exists $w0 ] } {
        set w0 ".dialog[incr count]"
    }

    toplevel $w0

    wm iconname $w0 "Select"
    wm iconbitmap $w0 @${FSLDIR}/tcl/fmrib.xbm

    wm title $w0 "Select subjects' high resolution images"

    frame $w0.f

#}}}
    #{{{ setup file selections

    set i 1
    while { $i <= $fmri(multiple) } {

	FSLFileEntry $w0.filename$i \
		-variable highres_files($i) \
		-label " $i:   " \
		-pattern "*.hdr" \
		-directory $PWD \
		-title "Select a high resolution image" \
		-width 60 \
		-filterhist VARS(history)

	pack $w0.filename$i -in $w0.f -side top -expand yes -fill both -padx 3 -pady 3
	incr i 1
    }

#}}}
    #{{{ setup buttons

frame $w0.btns
frame $w0.btns.b -relief raised -borderwidth 1

button $w0.cancel -command "feat4:multiple_regcheck $w 1; destroy $w0" -text "OK"

pack $w0.btns.b -side bottom -fill x -padx 3 -pady 5
pack $w0.cancel -in $w0.btns.b -side left -expand yes -padx 3 -pady 3 -fill y

#}}}
 
    pack $w0.f $w0.btns -expand yes -fill both
}

proc feat4:multiple_regcheck { w load } {
    global FSLDIR highres_files fmri

    if { $fmri(reg_yn) } {

	if { $fmri(multiple) < 2 } {
	    set nhighres 1
	} else {
	    set nhighres $fmri(multiple)
	}

	set AllOK 1

	if { [ exec ${FSLDIR}/bin/avwnvols [ file rootname $fmri(regstandard) ] 2> /dev/null ] < 1 } {
	    set AllOK 0
	}

	set i 1
	while { $i <= $nhighres } {
	    if { ![ info exists highres_files($i) ] || [ exec ${FSLDIR}/bin/avwnvols [ file rootname $highres_files($i) ] 2> /dev/null ] < 1 } {
		set AllOK 0
	    }
	    incr i 1
	}
    
	if { ! $AllOK} {
	    MxPause "Warning: you haven't filled in all the high resolution image selections with valid filenames"
	    return 1
	}
    }

    return 0
}

#}}}
#{{{ setup model

proc feat4:setup_model { w } {
    global FSLDIR fmri VARS PWD

    #{{{ setup window

    if { [ info exists fmri(w_model) ] } {
	if { [ winfo exists $fmri(w_model) ] } {
	    return 0
	}
    }

    set count 0
    set w0 ".dialog[incr count]"
    while { [ winfo exists $w0 ] } {
        set w0 ".dialog[incr count]"
    }

    set fmri(w_model) $w0

    toplevel $w0

    wm title $w0 "General Linear Model"
    wm iconname $w0 "GLM"
    wm iconbitmap $w0 @${FSLDIR}/tcl/fmrib.xbm

    frame $w0.f
    pack $w0.f -expand yes -fill both -in $w0 -side top

    canvas $w0.f.viewport -yscrollcommand "$w0.f.ysbar set" -xscrollcommand "$w0.xsbar set"
    scrollbar $w0.xsbar -command "$w0.f.viewport xview" -orient horizontal
    scrollbar $w0.f.ysbar -command "$w0.f.viewport yview" -orient vertical
    frame $w0.f.viewport.f
    $w0.f.viewport create window 0 0 -anchor nw -window $w0.f.viewport.f
    bind $w0.f.viewport.f <Configure> "feat4:scrollform_resize $w0 $w0.f.viewport"
    pack $w0.f.viewport -side left -fill both -expand true -in $w0.f

    set NB $w0.f.viewport.f.nb
    tixNoteBook $NB
    if { $fmri(analysis) != 4 } {
	$NB add evs -label "EVs"
    }
    $NB add contrasts -label "Contrasts"
    $NB add ftests -label "F-tests"
    pack $NB -in $w0.f.viewport.f

#}}}
    #{{{ setup F-tests

set fmri(ftestsf) [ $NB subwidget ftests ]

frame $fmri(ftestsf).fgrid

tixControl $fmri(ftestsf).nftests -label " Number of F-tests " \
        -variable fmri(nftests) -step 1 -min 0 -command "feat4:setup_model_update_ftests $w $fmri(ftestsf).fgrid"

pack $fmri(ftestsf).nftests $fmri(ftestsf).fgrid -in $fmri(ftestsf) -side top -anchor w -padx 5 -pady 2

feat4:setup_model_update_ftests $w $fmri(ftestsf).fgrid 0

$w.bhelp bind $fmri(ftestsf).nftests -msg "F-tests enable you to investigate several contrasts at the same time,
for example to see whether any of them (or any combination of them) is
significantly non-zero. This is thus a \"generalisation\" of the
T-test where only one contrast is tested. Also, the F-test allows you
to compare the contribution of each contrast to the model and decide
on significant and non-significant ones.

One example of F-test usage is if a particular stimulation is to be
represented by several EVs, each with the same input function
(e.g. square wave or custom timing) but all with different HRF
convolutions - i.e. several \"basis functions\". Putting all relevant
resulting parameter estimates together into an F-test allows the
complete fit to be tested against zero without having to specify the
relative weights of the basis functions (as one would need to do with
a single contrast). So - if you had three basis functions (EVs 1,2 and
3) the wrong way of combining them is a single (T-test) contrast of
\[1 1 1\]. The right way is to make three contrasts \[1 0 0\] \[0 1 0\] and
\[0 0 1\] and enter all three contrasts into an F-test.

You can carry out as many F-tests as you like. Each test includes the
particular contrasts that you specify by clicking on the appropriate
buttons."

#}}}
    #{{{ setup contrasts

set fmri(contrastsf) [ $NB subwidget contrasts ]

frame $fmri(contrastsf).congrid

tixControl $fmri(contrastsf).ncon -label " Number of contrasts " \
        -variable fmri(ncon) -step 1 -min 1 -command "feat4:setup_model_update_contrasts $w $fmri(contrastsf).congrid"

pack $fmri(contrastsf).ncon $fmri(contrastsf).congrid -in $fmri(contrastsf) -side top -anchor w -padx 5 -pady 2

feat4:setup_model_update_contrasts $w $fmri(contrastsf).congrid 0

$w.bhelp bind $fmri(contrastsf).ncon -msg "

Each EV (explanatory variable, i.e., waveform) in the design matrix
results in a PE (parameter estimate) image.  This estimate tells you
how strongly that waveform fit the data at each voxel - ths higher it
is, the better the fit. For an unblurred square wave input (which will
be scaled in the model from -1 to 1), the PE image is equivalent to
0.5* the \"mean difference image\". To convert from a PE to a t
statistic image, the PE is divided by it's standard error, which is
derived from the residual noise after the complete model has been
fit. The t image is then transformed into a Z statistic via standard
statistical transformation. As well as Z images arising from single
EVs, it is possible to combine different EVs (waveforms) - for
example, to see where one has a bigger effect than another. To do
this, one PE is subtracted from another, a combined standard error is
calculated, and a new Z image is created.

All of the above is controlled by you, by setting up contrasts. Each
output Z statistic image is generated by setting up a contrast vector;
thus set the number of outputs that you want, using \"Number of
contrasts\". To convert a single EV into a Z statistic image, set it's
contrast value to 1 and all others to 0. Thus the simplest design,
with one EV only, has just one contrast vector, and only one entry in
this contrast vector; 1. To add more contrast vectors, increase the
\"Number of contrasts\". To compare two EVs, for example, to subtract
one stimulus type (EV1) from another type (EV2), set EV1's contrast
value to -1 and EV2's to 1. A Z statistic image will be generated
according to this request."

#}}}
    #{{{ setup EVs

if { $fmri(analysis) != 4 } {

    set fmri(evsf) [ $NB subwidget evs ]

    tixControl $fmri(evsf).evs -label " Number of EVs " -variable fmri(evs) -step 1 -min 1 -command "feat4:setup_model_update_allevs $w $fmri(evsf)"

    $w.bhelp bind $fmri(evsf).evs -msg "The basic number of explanatory variables in the design matrix; this
means the number of different effects that you wish to model - one for
each modelled stimulation type, and one for each modelled confound."

    tixNoteBook $fmri(evsf).evsnb

    feat4:setup_model_update_allevs $w $fmri(evsf) 0

    pack $fmri(evsf).evs $fmri(evsf).evsnb -in $fmri(evsf) -padx 2 -pady 2 -side top -anchor w
}

#}}}
    #{{{ setup buttons

frame $w0.btns
pack $w0.btns -padx 2 -pady 2 -in $w0 -side bottom

frame $w0.btns.b -relief raised -borderwidth 2
pack $w0.btns.b -in $w0.btns -side bottom -fill x -padx 2 -pady 2

button $w0.btns.b.view -command "feat4:setup_model_preview $w" -text "View design"
button $w0.btns.b.acview -command "feat4:setup_model_acpreview $w" -text "Covariance"
button $w0.btns.b.cancel -command "feat4:setup_model_destroy $w $w0" -text "Done"
pack $w0.btns.b.view $w0.btns.b.acview $w0.btns.b.cancel -in $w0.btns.b -side left -expand yes -padx 3 -pady 3 -fill y

#}}}
}

#{{{ feat4:scrollform_resize

proc feat4:scrollform_resize { w0 viewport } {

    set fwidth  [ winfo width $viewport.f ]
    if { $fwidth > 700 } {
	set fwidth 700
	pack $w0.xsbar -fill x -in $w0 -after $w0.f -before $w0.btns
    } else {
	pack forget $w0.f.xsbar
    }

    set fheight [ winfo height $viewport.f ]
    if { $fheight > 500 } {
	set fheight 500
	pack $w0.f.ysbar -side right -fill y -in $w0.f
    } else {
	pack forget $w0.f.ysbar
    }

    $viewport configure -width $fwidth -height $fheight -scrollregion [ $viewport bbox all ]
}

#}}}
#{{{ feat4:setup_model_vars_simple

proc feat4:setup_model_vars_simple { w } {
    global fmri

    set fmri(filmsetup) 1

    if { $fmri(wizard_type) == 1 } {

	set fmri(evs) 1

	set fmri(shape1) 0
	set fmri(skip1) 0
	set fmri(off1) $fmri(a_count)
	set fmri(on1) $fmri(b_count)
	set fmri(phase1) 0
	set fmri(stop1) -1
	set fmri(convolve1) 2
	set fmri(gammasigma1) 3
	set fmri(gammadelay1) 6
	set fmri(tempfilt_yn1) 1
	set fmri(deriv_yn1) 1
	
	set fmri(ncon) 1
	set fmri(con1.1) 1
	set fmri(conpic.1) 1

	set fmri(nftests) 0

	set fmri(paradigm_hp) [ expr 1.5 * ( $fmri(a_count) + $fmri(b_count) ) ]

    } else {

	set fmri(evs) 2

	set fmri(shape1) 0
	set fmri(shape2) 0
	set fmri(skip1) 0
	set fmri(skip2) 0
	set fmri(off1) [ expr $fmri(a_count) * 2 + $fmri(c_count) ]
	set fmri(on1) $fmri(b_count)
	set fmri(off2) [ expr $fmri(a_count) * 2 + $fmri(b_count) ]
	set fmri(on2) $fmri(c_count)
	set fmri(phase1) [ expr $fmri(a_count) + $fmri(c_count) ]
	set fmri(phase2) 0
	set fmri(stop1) -1
	set fmri(stop2) -1
	set fmri(convolve1) 2
	set fmri(convolve2) 2
	set fmri(gammasigma1) 3
	set fmri(gammasigma2) 3
	set fmri(gammadelay1) 6
	set fmri(gammadelay2) 6
	set fmri(tempfilt_yn1) 1
	set fmri(tempfilt_yn2) 1
	set fmri(deriv_yn1) 1
	set fmri(deriv_yn2) 1

	set fmri(ortho2.1) 0
	
	set fmri(ncon) 4
	set fmri(con1.1) 1
	set fmri(con1.2) 0
	set fmri(con2.1) 0
	set fmri(con2.2) 1
	set fmri(con3.1) 1
	set fmri(con3.2) -1
	set fmri(con4.1) -1
	set fmri(con4.2) 1

	set fmri(conpic.1) 1
	set fmri(conpic.2) 1
	set fmri(conpic.3) 1
	set fmri(conpic.4) 1

	set fmri(nftests) 1
	set fmri(ftest1.1) 1
	set fmri(ftest1.2) 1
	set fmri(ftest1.3) 0
	set fmri(ftest1.4) 0

	set fmri(paradigm_hp) [ expr 1.5 * ( ( 2 * $fmri(a_count) ) + $fmri(b_count) + $fmri(c_count) ) ]

    }
}

#}}}
#{{{ feat4:setup_model_init_evs

proc feat4:setup_model_init_evs { w } {
    global fmri
    
    for { set i 1 } { $i <= $fmri(evs) } { incr i 1 } {
	#{{{ initialise variables

if { ! [ info exists fmri(shape$i) ] } {
    set fmri(shape$i) 0
}

if { ! [ info exists fmri(skip$i) ] } {
    set fmri(skip$i) 0
}

if { ! [ info exists fmri(off$i) ] } {
    set fmri(off$i) 30
}

if { ! [ info exists fmri(on$i) ] } {
    set fmri(on$i) 30
}

if { ! [ info exists fmri(phase$i) ] } {
    set fmri(phase$i) 0
}

if { ! [ info exists fmri(stop$i) ] } {
    set fmri(stop$i) -1
}

if { ! [ info exists fmri(period$i) ] } {
    set fmri(period$i) 60
}

if { ! [ info exists fmri(convolve$i) ] } {
    set fmri(convolve$i) 2
}

if { ! [ info exists fmri(gausssigma$i) ] } {
    set fmri(gausssigma$i) 2.8
}

if { ! [ info exists fmri(gaussdelay$i) ] } {
    set fmri(gaussdelay$i) 5
}

if { ! [ info exists fmri(gammasigma$i) ] } {
    set fmri(gammasigma$i) 3
}

if { ! [ info exists fmri(gammadelay$i) ] } {
    set fmri(gammadelay$i) 6
}

if { ! [ info exists fmri(tempfilt_yn$i) ] } {
    set fmri(tempfilt_yn$i) 1
}

if { ! [ info exists fmri(deriv_yn$i) ] } {
    set fmri(deriv_yn$i) 1
}

if { ! [ info exists fmri(interactions${i}.1) ] && $i > 2 } {
    for { set j 1 } { $j < $i } { incr j 1 } {
	if { $j < 3 } {
	    set fmri(interactions${i}.$j) 1
	} else {
	    set fmri(interactions${i}.$j) 0
	}
    }
}

#}}}
    }

    for { set i 1 } { $i <= $fmri(ncon) } { incr i 1 } {
	if { ! [ info exists fmri(conpic.$i) ] } {
	    set fmri(conpic.$i) 1
	}
    }
}

#}}}
#{{{ feat4:setup_model_update_allevs

proc feat4:setup_model_update_allevs { w w0 dummy } {
    global fmri PWD

    feat4:setup_model_init_evs $w

    for { set i 1 } { $i > 0 } { incr i 1 } {
	if { $i <= $fmri(evs) } {
	    if { ! [ info exists fmri(modelf$i) ] || ! [ winfo exists $fmri(modelf$i) ] } {
		#{{{ setup EV tab i

$w0.evsnb add ev$i -label "EV $i"
set fmri(modelf$i) [ $w0.evsnb subwidget ev$i ]

#{{{ basic shape

    set grot $fmri(shape$i)

    tixOptionMenu $w0.evsnb.shape$i -label " Basic shape: " -variable fmri(shape$i)
    $w0.evsnb.shape$i add command 0 -label "Square"
    $w0.evsnb.shape$i add command 1 -label "Sinusoid"
    $w0.evsnb.shape$i add command 2 -label "Custom (1 entry per volume)"
    $w0.evsnb.shape$i add command 3 -label "Custom (3 column format)"
    if { $i > 2 } {
	$w0.evsnb.shape$i add command 4 -label "Interaction"
    }

    set fmri(shape$i) $grot

$w.bhelp bind $w0.evsnb.shape$i -msg "Choose the basic shape of the waveform that describes the stimulus or
confound that you wish to model. The basic waveform should be exactly
in time with the applied stimulation, i.e., not lagged at all; the
measured (time-series) response will be delyed with respect to the
stimulation, and this delay is modelled in the design matrix by
convolution of the basic waveform with a suitable haemodynamic
response function (see appropriate bubble-help).

For an on/off (or a regularly-spaced single-event) experiment this
would normally be a square wave, controlled by parameters which also
need setting up. To model single-event experiments with this method,
the \"On\" periods will probably be small - e.g., 1s or even less.

For a single-event experiment with irregular timing for the
stimulations, a custom file can be used. With \"Custom (1 entry per
volume)\", the custom file should be ASCII, and should be a list of
numbers, separated by spaces or newlines, with one number for each
volume (after subtracting the number of deleted images). These numbers
can either all be 0s and 1s, or can take a range of values. The former
case would be appropriate if the same stimulus was applied at varying
time points; the latter would be appropriate, for example, if recorded
subject responses are to be inserted as an effect to be modelled. Note
that it may or may not be appropriate to convolve this particular
waveform with an HRF - in the case of single-event, it is.

For even finer control over the input waveform, choose \"Custom (3
column format)\". In this case the custom file consists of triplets of
numbers; you can input any number of triplets. Each triplet describes
a short period of time and the value of the input function during that
time. The first number in each triplet is the onset (in seconds) of
the period, the second number is the duration (in seconds) of the
period, and the third number is the value of the input during that
period. The same comments as above apply, about whether these numbers
are 0s and 1s, or vary continuously.

Note that whilst all columns are demeaned before model fitting,
neither custom input will be rescaled. One reason is that if you have
more than one custom waveform, you may want to keep relative scalings
in the set values.

If you select \"Interaction\" then you can set which EVs this EV
models the interaction between. This EV is produced by multiplying
together selected EVs, and allows the modelling of the non-additive
interaction between the selected EVs."

#}}}
#{{{ timings / custom name

frame $w0.evsnb.timings$i

tixControl $w0.evsnb.skip$i -label "    Skip (s)" -variable fmri(skip$i) -step 1 -min 0 -selectmode immediate
$w.bhelp bind $w0.evsnb.skip$i -msg "The initial period (seconds) before the waveform commences."

tixControl $w0.evsnb.off$i -label "    Off (s)" -variable fmri(off$i) -step 1 -min 0 -selectmode immediate
$w.bhelp bind $w0.evsnb.off$i -msg "The duration (seconds) of the \"Off\" periods in the square wave."

tixControl $w0.evsnb.on$i -label "    On (s)" -variable fmri(on$i) -step 1 -min 0 -selectmode immediate
$w.bhelp bind $w0.evsnb.on$i -msg "The duration (seconds) of the \"On\" periods in the square wave."

tixControl $w0.evsnb.phase$i -label "    Phase (s)" -variable fmri(phase$i) -step 1 -min 0 -selectmode immediate
$w.bhelp bind $w0.evsnb.phase$i -msg "The phase shift (seconds) of the waveform. By default, after the \"Skip\" period, the square wave\nstarts with a full \"Off\" period and the \"Sinusoid\" starts by falling from zero. However, the\nwave can be brought forward in time according to the phase shift."

tixControl $w0.evsnb.stop$i -label "    Stop after (s)" -variable fmri(stop$i) -step 1 -min -1 -selectmode immediate
$w.bhelp bind $w0.evsnb.stop$i -msg "The active duration (seconds) of the waveform, starting\nafter the \"Skip\" period. \"-1\" means do not stop."

tixControl $w0.evsnb.period$i -label "    Period (s)" -variable fmri(period$i) -step 1 -min 0 -selectmode immediate
$w.bhelp bind $w0.evsnb.period$i -msg "The period (seconds) of the \"Sinusoid\" waveform."

FSLFileEntry $w0.evsnb.custom$i \
	-variable fmri(custom$i) \
	-pattern "*" \
	-directory $PWD \
	-label "  Filename" \
	-title "Select an event file" \
	-width 30 \
	-filterhist VARS(history)

if { $i > 2 } {
    frame $w0.evsnb.interaction$i
    label $w0.evsnb.interaction$i.label -text "Between EVs "
    pack $w0.evsnb.interaction$i.label -in $w0.evsnb.interaction$i -side left -padx 3
    for { set j 1 } { $j < $i } { incr j 1 } {
	label $w0.evsnb.interaction$i.label$j -text "$j"
	checkbutton $w0.evsnb.interaction$i.$j -variable fmri(interactions${i}.$j) -command "feat4:setup_model_update_evs $w $w0 $i 0 0"
	pack $w0.evsnb.interaction$i.label$j $w0.evsnb.interaction$i.$j -in $w0.evsnb.interaction$i -side left
    }
}

#}}}

collapsible frame $w0.evsnb.opt$i -title "Advanced"
#{{{ convolution

set grot $fmri(convolve$i)

tixOptionMenu $w0.evsnb.convolve$i -label " Convolution: " -variable fmri(convolve$i)
$w0.evsnb.convolve$i add command 0 -label "None"
$w0.evsnb.convolve$i add command 1 -label "Gaussian"
$w0.evsnb.convolve$i add command 2 -label "Gamma"
$w0.evsnb.convolve$i add command 3 -label "SPM99 HRF"

set fmri(convolve$i) $grot

$w.bhelp bind $w0.evsnb.convolve$i -msg "The form of the HRF (haemodynamic response function) convolution that will be applied to the basic\nwaveform. This blurs and delays the original waveform, in an attempt to match the difference\nbetween the input function (original waveform, i.e., stimulus waveform) and the output function\n(measured FMRI haemodynamic response). If the original waveform is already in an appropriate form,\ne.g., was sampled from the data itself, \"None\" should be selected. The other options are all\nsomewhat similar blurring and delaying functions. \"Gaussian\" is simply a Gaussian kernel, whose\nwidth and lag can be altered. \"Gamma\" is a Gamma variate (in fact a normalisation of the\nprobability density function of the Gamma function); again, width and lag can be altered.\n\"SPM99 HRF\" is a preset function which is a mixture of two Gamma functions - a standard positive\nfunction at normal lag, and a small, delayed, inverted Gamma, which attempts to model the late\nundershoot. Unless you understand the differences between the above options, and what effect the\nnumerical parameters have, it is safest to use the default settings."

#}}}
#{{{ convolution parameters

frame $w0.evsnb.convparams$i

tixControl $w0.evsnb.gausssigma$i -label "    Sigma (s)"    -variable fmri(gausssigma$i) -step 0.1 -min 0 -selectmode immediate

tixControl $w0.evsnb.gaussdelay$i -label "    Peak lag (s)" -variable fmri(gaussdelay$i) -step 0.1 -min 0 -selectmode immediate

tixControl $w0.evsnb.gammasigma$i -label "    Stddev (s)"   -variable fmri(gammasigma$i) -step 0.1 -min 0.01 -selectmode immediate

tixControl $w0.evsnb.gammadelay$i -label "    Mean lag (s)" -variable fmri(gammadelay$i) -step 0.1 -min 0.01 -selectmode immediate

#}}}
#{{{ temporal filtering

frame $w0.evsnb.tempfilt$i

label $w0.evsnb.tempfilt$i.label -text "Apply temporal filtering"

checkbutton $w0.evsnb.tempfilt$i.yn -variable fmri(tempfilt_yn$i)

pack $w0.evsnb.tempfilt$i.label $w0.evsnb.tempfilt$i.yn -in $w0.evsnb.tempfilt$i -side left -padx 3

$w.bhelp bind $w0.evsnb.tempfilt$i -msg "You should normally apply the same temporal filtering to the model as
you have applied to the data, as the model is designed to look like
the data before temporal filtering was applied. Thus any extra
smoothing is correctly modelled, and long-time-scale components in the
model will be dealt with correctly."

#}}}
#{{{ temporal derivative

frame $w0.evsnb.deriv$i

label $w0.evsnb.deriv$i.label -text "Add temporal derivative"

checkbutton $w0.evsnb.deriv$i.yn -variable fmri(deriv_yn$i)

pack $w0.evsnb.deriv$i.label $w0.evsnb.deriv$i.yn -in $w0.evsnb.deriv$i -side left -padx 3

$w.bhelp bind $w0.evsnb.deriv$i -msg "Adding a fraction of the temporal derivative of the blurred original waveform is equivalent to\nshifting the waveform slightly in time, in order to achieve a slightly better fit to the data.\nThus adding in the temporal derivative of a waveform into the design matrix allows a better fit\nfor the whole model, reducing unexplained noise, and increasing resulting statistical significances.\nNote that it is not possible to use the temporal derivative in the parameter estimates - thus in the\ngraphical design matrix display, temporal derivatives always have zero weights in the contrast vectors."

#}}}
#{{{ orthogonalise

if { $i > 1 } {
    frame $w0.evsnb.ortho$i

    if { $i == 2 } {
	label $w0.evsnb.ortho$i.label -text "Orthogonalise WRT EV"
    } else {
	label $w0.evsnb.ortho$i.label -text "Orthogonalise WRT EVs"
    }

    pack $w0.evsnb.ortho$i.label -in $w0.evsnb.ortho$i -side left -padx 3
    for { set j 1 } { $j < $i } { incr j 1 } {
	label $w0.evsnb.ortho$i.label$j -text "$j"
	checkbutton $w0.evsnb.ortho$i.$j -variable fmri(ortho${i}.$j)
	pack $w0.evsnb.ortho$i.label$j $w0.evsnb.ortho$i.$j -in $w0.evsnb.ortho$i -side left
    }

    $w.bhelp bind $w0.evsnb.ortho$i -msg "Orthogonalising an EV with respect to other EVs means that it is
completely independent of the other EVs, i.e. contains no component
related to them. Most sensible designs are already in this form - all
EVs are orthogonal to all others. However, this may not be the case;
you can use this facility to force an EV to be orthogonal to some or
all lower number EVs. This is achieved by subtracting from the EV that
part which is related to the other EVs selected here.

An example use would be if you had a lower numbered EV which was a
constant height spike train, and the current EV is derived from this
other one, but with a linear increase in spike height imposed, to
model an increase in response during the experiment for any
reason. You would not want the current EV to contain any component of
the constant height EV, so you would orthogonalise the current EV wrt
the other."

}

#}}}

pack $w0.evsnb.shape$i $w0.evsnb.timings$i $w0.evsnb.opt$i -in $fmri(modelf$i) -padx 5 -pady 2 -side top -anchor w
pack $w0.evsnb.convolve$i $w0.evsnb.convparams$i $w0.evsnb.tempfilt$i $w0.evsnb.deriv$i -in $w0.evsnb.opt$i.b -padx 5 -pady 2 -side top -anchor w
if { $i > 1 } {
    pack $w0.evsnb.ortho$i -in $w0.evsnb.opt$i.b -padx 5 -pady 2 -side top -anchor w
}
		
$w0.evsnb.shape$i configure  -command " feat4:setup_model_update_evs $w $w0 $i 1"
$w0.evsnb.convolve$i configure -command " feat4:setup_model_update_evs $w $w0 $i 0"
$w0.evsnb.tempfilt$i.yn configure -command "feat4:setup_model_update_evs $w $w0 $i 0 0"
feat4:setup_model_update_evs $w $w0 $i 0 0

#}}}
	    } else {
		$w0.evsnb pageconfigure ev$i -state normal
	    }
	} else {
	    if { [ info exists fmri(modelf$i) ] && [ winfo exists $fmri(modelf$i) ] } {
		$w0.evsnb pageconfigure ev$i -state disabled
	    } else {
		set i -10
	    }
	}
	
	feat4:setup_model_update_contrasts $w $fmri(contrastsf).congrid 0
    }
}    

#}}}
#{{{ feat4:setup_model_update_evs

proc feat4:setup_model_update_evs { w w0 i force dummy } {
    global fmri

    pack forget $w0.evsnb.skip$i $w0.evsnb.off$i $w0.evsnb.on$i $w0.evsnb.phase$i $w0.evsnb.stop$i $w0.evsnb.period$i $w0.evsnb.custom$i $w0.evsnb.interaction$i

    if { $fmri(shape$i) == 0 } {
	pack $w0.evsnb.skip$i $w0.evsnb.off$i $w0.evsnb.on$i $w0.evsnb.phase$i $w0.evsnb.stop$i -in $w0.evsnb.timings$i -padx 5 -pady 2 -side top -anchor w
    } elseif { $fmri(shape$i) == 1 } {
	pack $w0.evsnb.skip$i $w0.evsnb.period$i $w0.evsnb.phase$i $w0.evsnb.stop$i -in $w0.evsnb.timings$i -padx 5 -pady 2 -side top -anchor w
    } elseif { $fmri(shape$i) == 2 || $fmri(shape$i) == 3 } {
	pack $w0.evsnb.custom$i -in $w0.evsnb.timings$i -padx 5 -pady 2 -side top -anchor w
    } elseif { $fmri(shape$i) == 4 } {
	pack $w0.evsnb.interaction$i -in $w0.evsnb.timings$i -padx 5 -pady 2 -side top -anchor w
	set selected 0
	for { set j 1 } { $j < $i } { incr j 1 } {
	    set selected [ expr $selected + $fmri(interactions${i}.$j) ]
	}
	if { $selected < 2 } {
	    if { !$fmri(interactions${i}.1) } {
		set fmri(interactions${i}.1) 1
	    } else {
		set fmri(interactions${i}.2) 1
	    }
	}
    }

    pack forget $w0.evsnb.gausssigma$i $w0.evsnb.gaussdelay$i $w0.evsnb.gammasigma$i $w0.evsnb.gammadelay$i

    if { $fmri(shape$i) == 4 } {
	set fmri(convolve$i) 0
	set fmri(tempfilt_yn$i) 0
    } else {
	if { $force } {
	    set fmri(convolve$i) 2
	    set fmri(tempfilt_yn$i) 1
	}
    }

    if { $fmri(convolve$i) == 1 } {
	pack $w0.evsnb.gausssigma$i $w0.evsnb.gaussdelay$i -in $w0.evsnb.convparams$i -padx 5 -pady 2 -side top -anchor w
    } elseif { $fmri(convolve$i) == 2 } {
	pack $w0.evsnb.gammasigma$i $w0.evsnb.gammadelay$i -in $w0.evsnb.convparams$i -padx 5 -pady 2 -side top -anchor w
    }
}

#}}}
#{{{ feat4:setup_model_update_contrasts

proc feat4:setup_model_update_contrasts { w w0 dummy } {
    global fmri

    for { set j 1 } { $j > 0 } { incr j 1 } {
	if { $j <= $fmri(evs) } {
	    if { ! [ winfo exists $w0.evlabel${j} ] } {
		label $w0.evlabel${j} -text "EV$j"
		grid $w0.evlabel${j} -in $w0 -row 0 -column [ expr $j + 1 ]
	    }
	} else {
	    if { [ winfo exists $w0.evlabel${j} ] } {
		destroy $w0.evlabel${j}
	    } else {
		set j -10
	    }
	}
    }

    for { set i 1 } { $i > 0 } { incr i 1 } {

	for { set j 1 } { $j > 0 } { incr j 1 } {
	    if { $i <= $fmri(ncon) && $j <= $fmri(evs) } {

		if { ! [ info exists fmri(con${i}.${j}) ] } {
		    set fmri(con${i}.${j}) 0
		    if { $i==1 && $j==1 } {
			set fmri(con${i}.${j}) 1
		    }
		}

		if { ! [ winfo exists $w0.con${i}_${j} ] } {
		    tixControl $w0.con${i}_${j} -variable fmri(con${i}.${j}) -step 1 -selectmode immediate -options { entry.width 3 }
		    grid $w0.con${i}_${j} -in $w0 -row $i -column [ expr $j + 1 ]
		    $w.bhelp bind $w0.con${i}_${j} -msg "The weight given to EV$j within contrast vector $i."
		}
	    } else {
		if { [ winfo exists $w0.con${i}_${j} ] } {
		    destroy $w0.con${i}_${j}
		} else {
		    set j -10
		}
	    }
	}

	if { $i <= $fmri(ncon) } {

	    if { ! [ info exists fmri(conpic.$i) ] } {
		set fmri(conpic.$i) 1
	    }

	    if { ! [ winfo exists $w0.label$i ] } {
		label $w0.label$i -text " C$i "
		grid $w0.label$i -in $w0 -row $i -column 0
		$w.bhelp bind $w0.label$i -msg "Contrast vector number $i - this will result in Z statistic image number $i"
		checkbutton $w0.conpic$i -variable fmri(conpic.$i)
		grid $w0.conpic$i -in $w0 -row $i -column 1
		$w.bhelp bind $w0.conpic$i -msg "Include contrast $i in web page report? (Turn off if
this contrast is only to be used within F-tests.)"
	    }
	} else {
	    if { [ winfo exists $w0.label$i ] } {
		destroy $w0.label$i
		destroy $w0.conpic$i
	    } else {
		set i -10
	    }
	}
	
    }

    feat4:setup_model_update_ftests $w $fmri(ftestsf).fgrid 0
}

#}}}
#{{{ feat4:setup_model_update_ftests

proc feat4:setup_model_update_ftests { w w0 dummy } {
    global fmri

    for { set j 1 } { $j > 0 } { incr j 1 } {
	if { $fmri(nftests) > 0 && $j <= $fmri(ncon) } {
	    if { ! [ winfo exists $w0.clabel$j ] } {
		label $w0.clabel${j} -text "C$j"
		grid $w0.clabel${j} -in $w0 -row 0 -column $j
	    }
	} else {
	    if { [ winfo exists $w0.clabel$j ] } {
		destroy $w0.clabel${j}
	    } else {
		set j -10
	    }
	}
    }

    for { set i 1 } { $i > 0 } { incr i 1 } {

	for { set j 1 } { $j > 0 } { incr j 1 } {

	    if { ! [ info exists fmri(ftest${i}_${j}) ] } {
		set fmri(ftest${i}_${j}) 0
	    }

	    if { $i <= $fmri(nftests) && $j <= $fmri(ncon) } {
		if { ! [ winfo exists $w0.ftest${i}_${j} ] } {
		    checkbutton $w0.ftest${i}_${j} -variable fmri(ftest${i}.$j)
		    grid $w0.ftest${i}_${j} -in $w0 -row $i -column $j
		    $w.bhelp bind $w0.ftest${i}_${j} -msg "Include contrast vector $j in F-test $i?"
		}
	    } else {
		if { [ winfo exists $w0.ftest${i}_${j} ] } {
		    destroy $w0.ftest${i}_${j}
		} else {
		    set j -10
		}
	    }
	}

	if { $i <= $fmri(nftests) } {
	    if { ! [ winfo exists $w0.flabel$i ] } {
		label $w0.flabel$i -text " F$i "
		grid $w0.flabel$i -in $w0 -row $i -column 0
		$w.bhelp bind $w0.flabel$i -msg "F-test number $i - this will result in F statistic image number $i"
	    }
	} else {
	    if { [ winfo exists $w0.flabel$i ] } {
		destroy $w0.flabel$i
	    } else {
		set i -10
	    }
	}
    }
}

#}}}
#{{{ feat4:setup_model_preview

proc feat4:setup_model_preview { w } {
    global fmri

    set fmri(filmsetup) 1

    set problem [ feat4:write $w 1 $fmri(feat_filename) ]
    if { $problem } {
	return 1
    }

    set count 0
    set w1 ".dialog[incr count]"
    while { [ winfo exists $w1 ] } {
        set w1 ".dialog[incr count]"
    }

    toplevel $w1 -visual truecolor
    wm title $w1 "Model"
    wm iconname $w1 "Model"

    set graphpic [ image create photo -file [ file rootname $fmri(feat_filename) ].ppm ]

    button $w1.btn -command "destroy $w1" -image $graphpic -borderwidth 0
    pack $w1.btn -in $w1

    $w.bhelp bind $w1.btn -msg "This is a graphical representation of the design matrix and parameter
contrasts.

The bar on the left is a representation of time, which starts at the
top and points downwards. The white marks show the position of every
10th volume in time. The red bar shows the period of the longest
temporal cycle which was passed by the highpass filtering.

The main top part shows the design matrix; time is represented on the
vertical axis and each column is a different explanatory variable
(e.g., stimulus type). Both the red lines and the black-white images
represent the same thing - the variation of the waveform in time.

Below this is shown the requested contrasts; each column refers to the
weighting of the relevant explanatory variable (often either just 1, 0
or -1), and each row is a different contrast vector. Thus each row
will result in it's own Z statistic image.

If F-tests have been specified, these appear to the right of the
contrasts; each column is a different F-test, with the inclusion of
particular contrasts depicted by a red square instead of a black one.

Click anywhere to dismiss this window."

  return 0
}

#}}}
#{{{ feat4:setup_model_acpreview

proc feat4:setup_model_acpreview { w } {
    global fmri

    if { $fmri(evs) < 2 && $fmri(deriv_yn1) == 0 } {
	return 0
    }

    set problem [ feat4:write $w 1 $fmri(feat_filename) ]
    if { $problem } {
	return 1
    }

    set count 0
    set w1 ".dialog[incr count]"
    while { [ winfo exists $w1 ] } {
        set w1 ".dialog[incr count]"
    }

    toplevel $w1 -visual truecolor
    wm title $w1 "Covariance matrix"
    wm iconname $w1 "Covariance"

    set graphpic [ image create photo -file [ file rootname $fmri(feat_filename) ]_cov.ppm ]

    button $w1.btn -command "destroy $w1" -image $graphpic -borderwidth 0
    pack $w1.btn -in $w1

    $w.bhelp bind $w1.btn -msg "This is a graphical representation of the covariance matrix of the
design matrix.

The first matrix shows the (absolute value) of the normalised
correlation of each EV with each EV. If a design is well-conditioned
(i.e. not approaching rank deficiency) then the diagonal elements
should be white and all others darker.

So - if there are any very bright elements off the diagonal, you can
immediately tell which EVs are too similar to each other - for
example, if element \[1,3\] (and \[3,1\]) is bright then columns 1 and
3 in the design matrix are possibly too similar.

Note that this includes all final EVs, including any added temporal
derivatives.

The second matrix shows a similar thing after the design matrix has
been run through SVD (singular value decomposition). All non-diagonal
elements will be zero and the diagonal elements are given by the
eigenvalues of the SVD, so that a poorly-conditioned design is obvious
if any of the diagonal elements are black.

Click anywhere to dismiss this window."

  return 0
}

#}}}
#{{{ feat4:setup_model_destroy

proc feat4:setup_model_destroy { w w0 } {
    global fmri

    if { ! [ feat4:setup_model_preview $w ] } {
	destroy $w0
    }
}

#}}}

#}}}
#{{{ feat4:medxload

proc feat4:medxload { featdir } {
    global PXHOME FSLDIR VARS argc argv PWD entries loadtimeseries

    #{{{ setup main window

set count 0
set w ".dialog[incr count]"
while { [ winfo exists $w ] } {
    set w ".dialog[incr count]"
}

toplevel $w

wm title      $w "Load FEAT results into MEDx"
wm iconname   $w "FEAT Load"
wm iconbitmap $w @${FSLDIR}/tcl/fmrib.xbm

frame $w.f
pack $w.f -in $w -side top

if { $featdir != 0 } {
    set entries($w,1) $featdir
}

FSLFileEntry $w.f.featdir \
	-variable entries($w,1) \
	-pattern "*.feat" \
	-directory "~" \
	-label "Select a FEAT directory" \
	-title "Select a FEAT directory" \
	-width 20 \
	-dirasfile "design.fsf" \
	-filterhist VARS(history)

frame $w.f.timeseries
label $w.f.timeseries.label -text "Load timeseries data"
set loadtimeseries 1
checkbutton $w.f.timeseries.yn -variable loadtimeseries
pack $w.f.timeseries.label $w.f.timeseries.yn -in $w.f.timeseries -side left -anchor w

pack $w.f.featdir $w.f.timeseries -in $w.f -padx 5 -pady 5 -anchor w

#}}}
    #{{{ Button Frame

frame $w.btns
frame $w.btns.b -relief raised -borderwidth 1
    
button $w.apply -command "feat4:run_medxload_proc $w" -text "Go"

button $w.cancel -command "destroy $w" -text "Exit"

pack $w.btns.b -side bottom -fill x -padx 3 -pady 3
pack $w.apply $w.cancel -in $w.btns.b -side left -expand yes -padx 3 -pady 3 -fill y

pack $w.btns -in $w -side bottom -fill x

#}}}
}

proc feat4:run_medxload_proc { w } {
    global PXHOME FSLDIR entries loadtimeseries
    feat4:medxload_proc $entries($w,1) $loadtimeseries
}

proc feat4:medxload_proc { FD loadtimeseries } {
    #{{{ setup

    global PXHOME FSLDIR

    puts "Loading $FD"

    MxNewFolder folder window

    MxSetFolderProperties $folder "{Name $FD}"

if { ! [ file exists ${FD}/filtered_func_data.hdr ] } {
    set loadtimeseries 0
}

#}}}
    #{{{ load lowres files

catch { exec sh -c "echo ${FD}/example_func.hdr ${FD}/mask.hdr ${FD}/stats/zstat*hdr ${FD}/stats/fixed*hdr ${FD}/stats/random*hdr ${FD}/cluster_mask_*hdr ${FD}/thresh_*hdr ${FD}/Rendered_thresh_*.hdr" } lowreslist

set lowrespagelist ""
foreach lowres $lowreslist {
    if { [ file exists $lowres ] } {
	MxOpenImage $folder $lowres lowrespage
	set lowrespagelist "$lowrespage $lowrespagelist"
    }
}

#}}}
    if { $loadtimeseries } {
	#{{{ load func data

    MxOpenImage $folder ${FD}/filtered_func_data.hdr Group

    MxGetPagesFromGroup $Group GPages
    set total_volumes [ llength $GPages ]

    set AList ""
    lappend AList [ lindex $GPages 0 ]
    lappend AList [ lindex $GPages 1 ]
    MxGroupVolumes $AList "A" A

    set BList ""
    set i 2
    while { $i < $total_volumes } {
	lappend BList [ lindex $GPages $i ]
	incr i 1
    }
    MxGroupVolumes $BList "B" B

    MxBetweenGrpStatistics $A $B U blah blah2 blah3 FinalZscore ResultGroup

    MxGetPagesFromGroup $ResultGroup RP
    set RPList " [ lindex $RP 0 ] [ lindex $RP 1 ] [ lindex $RP 2 ] [ lindex $RP 3 ] [ lindex $RP 4 ] [ lindex $RP 6 ] [ lindex $RP 7 ] "
    MxDeletePage { $RPList }
    MxDeleteGroupPages { $A $B $ResultGroup } No

    MxSetTimeSeriesProperties $Group 1 red

#}}}
    }
    #{{{ process lowres files

foreach lowrespage $lowrespagelist {
    MxSetCurrentPage $lowrespage
    MxGetImageProperties $lowrespage props
    if { $loadtimeseries } {
	MxDupPage $FinalZscore zscore
	MxExpr $zscore = " $lowrespage "
	MxDeletePage $lowrespage
    } else {
	set zscore $lowrespage
    }
    MxComputeDisplayRange $zscore V Min Max
    set thefile [ file rootname [ keylget props File ] ]
    set thename [ keylget props Name ]
    if { $loadtimeseries } {
	name_page $zscore $thename
    }

    if { [ string first Rendered $thename ] == 0 } {
	set cal_min  [ exec $FSLDIR/bin/avwval $thefile cal_min ]
	set cal_max  [ exec $FSLDIR/bin/avwval $thefile cal_max ]
	set aux_file [ string trimright [ exec $FSLDIR/bin/avwval $thefile aux_file ] " " ]
	MxSetDisplayRange $zscore V $cal_min $cal_max
	MxLoadMaterialMap $zscore ${FSLDIR}/etc/luts/${aux_file}.map
	MxLoadLUT $zscore ${FSLDIR}/etc/luts/${aux_file}.lut
    }
}

if { $loadtimeseries } {
    MxDeletePage $FinalZscore
}

#}}}
}

#}}}
#{{{ feat4:exec

proc feat4:exec { thecommand } {

    global logout

    puts $logout $thecommand

    catch { exec sh -c $thecommand } errmsg
    
    if { $errmsg != "" } {
	puts $logout $errmsg
    } else {
	puts $logout ""
    }
}

#}}}
#{{{ feat4:connectivity

proc feat4:connectivity { image } {

    global FSLDIR

    set CONNECTIVITY "26"

#    maybe actually USE this test? but needs careful thought.......
#
#    set CONNECTIVITY "6"
#
#    if { [ expr abs([ exec $FSLDIR/bin/avwval $image pixdim3 ]) ] < 3.1 } {
#	set CONNECTIVITY "26"
#    }

    return $CONNECTIVITY
}

#}}}
#{{{ feat4:poststats

proc feat4:poststats { talspace } {

    global FSLDIR FD report ps rs logout fmri
    
    set allstatstypes "zstat zfstat fixed random"

    #{{{ setup Talairach/non-Tal thingies

set TAL      ""
set TALEXT   ""
set SLICER   "-A"
set TALSPACE ""

if { $talspace } {
    set TAL      "-TAL"
    set TALEXT   "_tal"
    set SLICER   "-S 2"
    set TALSPACE "--mm"
}

#}}}
    #{{{ setup fmri(zmin) and fmri(zmax)

# set to to initial extremes if doing autoscale
if { $fmri(zdisplay) == 0 } {
    set fmri(zmin) 1000
    set fmri(zmax) -1000
}

#}}}
    #{{{ thresholding

    set firsttime 1
    foreach statstypes $allstatstypes {

	for { set i 1 } {  $i < 1000 } { incr i 1 } {

	    if { [ file exists ${FD}/stats/$statstypes${i}.hdr ] } {
		set MaximumZ($i) 0
		if { [ string compare $statstypes "zstat" ] != 0 || $fmri(conpic.$i) == 1 } {
		    #{{{ mask Z stats and find max and smoothness

set zin  ${FD}/stats/$statstypes${i}
set zout ${FD}/thresh_$statstypes${i}

feat4:exec "$FSLDIR/bin/avwmaths $zin -mul ${FD}/mask $zout"

catch { exec ${FSLDIR}/bin/avwstats $zout -R } minmax
set MaximumZ($i) [ lindex $minmax 1 ]
puts $logout "MaximumZ($i)=$MaximumZ($i)"

if { $fmri(thresh) > 1 } {
    if { [ file exists ${FD}/stats/smoothness ] } {
	set DLH     [exec grep DLH    ${FD}/stats/smoothness | awk {{ print $2 }} - ]
	set VOLUME  [exec grep VOLUME ${FD}/stats/smoothness | awk {{ print $2 }} - ]
	set RESELS  [exec grep RESELS ${FD}/stats/smoothness | awk {{ print $2 }} - ]
	set NRESELS [ expr int ( $VOLUME / $RESELS ) ]
    } else {
	feat4:exec "$FSLDIR/bin/smoothest -m ${FD}/mask -z $zin > ${FD}/stats/$statstypes${i}.smoothness"
	
	set DLH     [exec grep DLH    ${FD}/stats/$statstypes${i}.smoothness | awk {{ print $2 }} - ]
	set VOLUME  [exec grep VOLUME ${FD}/stats/$statstypes${i}.smoothness | awk {{ print $2 }} - ]
	set RESELS  [exec grep RESELS ${FD}/stats/$statstypes${i}.smoothness | awk {{ print $2 }} - ]
	set NRESELS [ expr int ( $VOLUME / $RESELS ) ]
	
	puts $logout "DLH=$DLH VOLUME=$VOLUME RESELS=$RESELS NRESELS=$NRESELS
	"
    }
}

#}}}
		    #{{{ Thresholding

if { $fmri(thresh) > 0 } {

    if { $firsttime == 1 } {
	set ps "$ps Z (Gaussianised T/F) statistic images were thresholded"
    }

    if { $fmri(thresh) < 3 } {
	#{{{ voxel-based thresholding

if { $firsttime == 1 } {
    if { $fmri(thresh) == 1 } {
	set ps "$ps at P=$fmri(prob_thresh) (uncorrected)."
	set zthresh [ exec sh -c "${FSLDIR}/bin/ptoz $fmri(prob_thresh) -2" ]
    } else {
	set ps "$ps using resel (corrected Bonferroni) thresholding with a corrected significance threshold of P=$fmri(prob_thresh) \[Friston 1995, Forman 1995\]."
	set rs "$rs\[Friston 1995\] K.J. Friston, A.P. Holmes, K.J. Worsley, J.P. Poline, C.D. Frith and R.S.J. Frackowiak. Statistical Parametric Maps in Functional Imaging: A General Linear Approach. Human Brain Mapping 1995:2(189-210).<br>
	\[Forman 1995\] S.D Forman, J.D. Cohen, M. Fitzgerald, W.F. Eddy, M.A. Mintun and D.C. Noll.
	Improved assessment of significant activation in functional magnetic resonance imaging (fMRI): Use of a cluster-size threshold. 
	Magnetic Resonance in Medicine 1995:33(636-647).<br> 
	"
	set zthresh [ exec sh -c "${FSLDIR}/bin/ptoz [ expr $fmri(prob_thresh) / $NRESELS ] -2" ]
    }
}

puts $logout "thresholding at Z=$zthresh
"

feat4:exec "$FSLDIR/bin/avwmaths $zout -thr $zthresh $zout"

#}}}
    } else {
	#{{{ cluster thresholding

if { $firsttime == 1 } {
    set ps "$ps using clusters determined by Z>$fmri(z_thresh) and a cluster significance threshold of P=$fmri(prob_thresh) \[Worsley 1992, Friston 1994, Forman 1995\]."
    set rs "$rs\[Friston 1992\] K.J. Friston, K.J. Worsley, R.S.J. Frakowiak, J.C. Mazziotta and A.C. Evans. Assessing the significance of
    focal activations using their spatial extent. Human Brain Mapping 1992:1(214-220).<br>
    \[Worsley 1992\] K.J. Worsley, A.C. Evans, S. Marrett and P. Neelin. A three-dimensional statistical analysis
    for CBF activation studies in human brain. Journal of Cerebral Blood Flow and Metabolism 1992:12(900-918).<br> 
    \[Forman 1995\] S.D Forman, J.D. Cohen, M. Fitzgerald, W.F. Eddy, M.A. Mintun and D.C. Noll.
    Improved assessment of significant activation in functional magnetic resonance imaging (fMRI): Use of a cluster-size threshold. 
    Magnetic Resonance in Medicine 1995:33(636-647).<br> 
    "
}

set COPE ""
if { [ file exists ${FD}/stats/cope$i ] } {
    set COPE "-c ${FD}/stats/cope$i"
}

if { $MaximumZ($i) > $fmri(z_thresh) } {
    feat4:exec "$FSLDIR/bin/cluster -i $zout $COPE -t $fmri(z_thresh) -p $fmri(prob_thresh) -d $DLH --volume=$VOLUME --othresh=$zout -o ${FD}/cluster_mask_${statstypes}$i --connectivity=[ feat4:connectivity $zout ] $TALSPACE > ${FD}/cluster_${statstypes}${i}${TALEXT}.txt"
    feat4:exec "$FSLDIR/bin/cluster2html $FD cluster_${statstypes}${i} $TAL"
} else {
    feat4:exec "$FSLDIR/bin/avwmaths $zout -mul 0 $zout"
}

#}}}
    }

    catch { exec ${FSLDIR}/bin/avwstats $zout -R } minmax
    set MaximumZ($i) [ lindex $minmax 1 ]
    puts $logout "MaximumZ($i)=$MaximumZ($i)"
}

#}}}
		    #{{{ Find group Z min and max

if { $MaximumZ($i) > 0 && $fmri(zdisplay) == 0 } {
    
    catch { exec ${FSLDIR}/bin/avwstats $zout -l 0.0001 -R } minmax
    set Minimum      [ lindex $minmax 0 ]
    set MaximumZ($i) [ lindex $minmax 1 ]

    set Minimum [ expr $Minimum - 0.0001 ]
    if { $Minimum < 0.0001 } {
	set Minimum 0.0001
    }

    if { $fmri(zmin) > $Minimum } {
	set fmri(zmin) $Minimum
    }

    if { $fmri(zmax) < $MaximumZ($i) } {
	set fmri(zmax) $MaximumZ($i)
    }
}

#}}}
		    set firsttime 0
		}
	    }
	}
    }

#}}}
    #{{{ rendering

    if { $fmri(thresh) > 0 } {
	puts $logout "Rendering using zmin=$fmri(zmin) zmax=$fmri(zmax)"

	set underlying ${FD}/example_func
	if { ! [ file exists ${underlying}.hdr ] } {
	    set underlying ${FD}/mean_highres
	}

	set firsttime 1
	foreach statstypes $allstatstypes {

	    for { set i 1 } {  $i < 1000 } { incr i 1 } {

		if { [ file exists ${FD}/thresh_${statstypes}$i.hdr ] } {
		    #{{{ Rendering

catch { exec ${FSLDIR}/bin/avwstats ${FD}/thresh_${statstypes}$i -R } minmax
set MaximumZ($i) [ lindex $minmax 1 ]

if { $MaximumZ($i) > 0 } {
    
    feat4:exec "$FSLDIR/bin/overlay $fmri(rendertype) 0 $underlying -a ${FD}/thresh_${statstypes}$i $fmri(zmin) $fmri(zmax) ${FD}/Rendered_thresh_${statstypes}$i"

    feat4:exec "${FSLDIR}/bin/slicer ${FD}/Rendered_thresh_${statstypes}$i $SLICER 750 ${FD}/Rendered_thresh_${statstypes}${i}.ppm"
    feat4:exec "${FSLDIR}/bin/convert ${FD}/Rendered_thresh_${statstypes}${i}.ppm ${FD}/Rendered_thresh_${statstypes}${i}.gif"
    feat4:exec "/bin/rm -f ${FD}/Rendered_thresh_${statstypes}${i}.ppm"

    if { $firsttime == 1 } {
	puts $report "<hr><H2>Thresholded activation images</H2>"
	set firsttime 0
    }

    if { $fmri(thresh) == 3 } {
	puts $report "<p>${statstypes} $i<br>
	<a href=\"cluster_${statstypes}${i}${TALEXT}.html\"><IMG BORDER=0 SRC=\"Rendered_thresh_${statstypes}${i}.gif\"></a>
	"
    } else {
	puts $report "<p>${statstypes} $i<br>
	<IMG BORDER=0 SRC=\"Rendered_thresh_${statstypes}${i}.gif\">
	"
    }
}

#}}}
		}
	    }
	}
    }

#}}}
}

#}}}
#{{{ feat4:proc

proc feat4:proc { fsfroot } {
    #{{{ setup and load fsf

global FSLDIR HOME HOSTNAME VERSION CLUSTERRSH logout fmri feat_files highres_files FD report ps rs logout
 
feat4:load -1 1 ${fsfroot}.fsf

#}}}

    if { $fmri(analysis) > 0 || $fmri(reg_yn) } {
	for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {
	    set thedate [ exec date ]
	    if { $fmri(analysis) > 0 } {
		#{{{ get file/dir name and setup FEAT directory

if { $fmri(analysis) == 4 } {

    set OldFD $feat_files($session)

    set FD [ new_filename $OldFD ]
    
    if { ! [ file writable [ file dirname $FD ] ] } {
	set FD [ new_filename ${HOME}/[ file tail $OldFD ] ]
    }

    file copy $OldFD $FD
    catch { exec sh -c "cd $FD ; rm -f Rendered_thresh_zstat* cluster_mask_zstat* cluster_zstat* design* prob_mask_zstat* report* thresh_zstat* tsplot* ; cd stats ; rm -f cope* neff* tstat* varcope* zstat*" } errmsg

} else {

    set FD [ new_filename [ file rootname $feat_files($session) ].feat ]

    if { ! [ file writable [ file dirname $FD ] ] } {
	set FD [ new_filename ${HOME}/[ file tail $FD ] ]
    }

    exec /bin/mkdir $FD

    set funcdata [ file rootname $feat_files($session) ]
    set fmri(dim3) [ exec $FSLDIR/bin/avwval $funcdata dim3 ]
}

exec /bin/cp ${fsfroot}.fsf ${FD}/design.fsf
if { $fmri(paradigm_type) == 3 } {
    if { [ file exists ${fsfroot}.mat ] && [ file exists ${fsfroot}.con ] && [ file exists ${fsfroot}.gif ] } {
	exec /bin/cp ${fsfroot}.mat ${FD}/design.mat
	exec /bin/cp ${fsfroot}.con ${FD}/design.con
	exec /bin/cp ${fsfroot}.gif ${FD}/design.gif
	catch { exec /bin/cp ${fsfroot}_cov.gif ${FD}/design_cov.gif }
	catch { exec /bin/cp ${fsfroot}.fts ${FD}/design.fts }
    } else {
	set result [ catch { exec ${FSLDIR}/bin/feat_model ${FD}/design } errmsg ]
	if {$result != 0} {
	    puts "Problem with processing the model: $errmsg"
	}
	exec sh -c "${FSLDIR}/bin/convert ${FD}/design.ppm ${FD}/design.gif"
	catch { exec sh -c "${FSLDIR}/bin/convert ${FD}/design_cov.ppm ${FD}/design_cov.gif" } errmsg
    }
}

set fsfroot ${FD}/design

#}}}
		#{{{ write starts of log and report files

set LOGFILE    ${FD}/report.log
set REPORTFILE ${FD}/report.html
set logout [ open $LOGFILE "w" ]
set report [ open $REPORTFILE "w" ]
fconfigure $logout -buffering none
fconfigure $report -buffering none

set tailpid [ exec tail -f $LOGFILE & ]
catch { exec sh -c "sleep 36000 ; kill -9 $tailpid" & } errmsg

puts $logout "Started FEAT at $thedate on $HOSTNAME"

puts $report "<HTML>

<TITLE>FEAT Report</TITLE>

<BODY BACKGROUND=\"file:${FSLDIR}/doc/images/fsl-bg\">

<hr><CENTER>
<H1>FEAT Report</H1>
$REPORTFILE<br>
$thedate
</CENTER>
"

set ps "<hr><H2>Analysis methods</H2>
<p>Analysis was carried out using FEAT, the FMRIB Easy Analysis Tool."

set rs "<p><b>References</b><br>
"

#}}}
		#{{{ check npts, delete images, make example_func

if { $fmri(analysis) != 4 } {

    # check npts
    set total_volumes [ exec ${FSLDIR}/bin/avwnvols $funcdata 2> /dev/null ]
    puts $logout "Total original volumes = $total_volumes"
    if { $total_volumes != $fmri(npts) } {
	puts $logout "Error - $funcdata has a different number of time points to that in FEAT setup"
	return 1
    }

    # delete images
    if { $fmri(ndelete) > 0 } {
	puts $logout "Deleting $fmri(ndelete) volume(s) - BE WARNED for future analysis!"
	set total_volumes [ expr $total_volumes - $fmri(ndelete) ]
	feat4:exec "${FSLDIR}/bin/avwroi $funcdata ${FD}/prefiltered_func_data $fmri(ndelete) $total_volumes"
	set funcdata ${FD}/prefiltered_func_data
    }

    # choose halfway image and copy to example_func
    set target_vol_number [ expr $total_volumes / 2 ]
    feat4:exec "${FSLDIR}/bin/avwroi $funcdata ${FD}/example_func $target_vol_number 1"

}

#}}}
		#{{{ pre-stats

if { $fmri(analysis) != 4 } {
    if { $fmri(filtering_yn) } {

	set ps "$ps The following pre-statistics processing was applied"

	#{{{ motion correction

#mc: 0=none 1=MCFLIRT

if { $fmri(mc) != 0 } {
    set ps "$ps; Motion correction using MCFLIRT \[Jenkinson 2001\]"
    set rs "$rs\[Jenkinson 2001\] M. Jenkinson and S.M. Smith. A Global Optimisation Method for Robust Affine Registration of Brain Images. Medical Image Analysis 2001:5(2)(143-156).<br>
    "

    feat4:exec "$CLUSTERRSH ${FSLDIR}/bin/mcflirt -in $funcdata -out ${FD}/prefiltered_func_data_mcf -plots -refvol $target_vol_number"
    set funcdata ${FD}/prefiltered_func_data_mcf
    feat4:exec "mv ${funcdata}.par ${FD}/mc.par"
    
    feat4:exec "echo \"set term pbm color ; set size 1,0.5 ; set title 'MCFLIRT estimated rotations (radians)' ; plot '${FD}/mc.par' using 1 title 'x' with lines , '${FD}/mc.par' using 2 title 'y' with lines , '${FD}/mc.par' using 3 title 'z' with lines\" | ${FSLDIR}/bin/gnuplot > ${FD}/mc_rot.ppm"
    feat4:exec "${FSLDIR}/bin/convert ${FD}/mc_rot.ppm ${FD}/mc_rot.gif"
    feat4:exec "/bin/rm -f ${FD}/mc_rot.ppm"
    
    feat4:exec "echo \"set term pbm color ; set size 1,0.5 ; set title 'MCFLIRT estimated translations (mm)' ; plot '${FD}/mc.par' using 4 title 'x' with lines , '${FD}/mc.par' using 5 title 'y' with lines , '${FD}/mc.par' using 6 title 'z' with lines\" | ${FSLDIR}/bin/gnuplot > ${FD}/mc_trans.ppm"
    feat4:exec "${FSLDIR}/bin/convert ${FD}/mc_trans.ppm ${FD}/mc_trans.gif"
    feat4:exec "/bin/rm -f ${FD}/mc_trans.ppm"

    puts $report "<hr><H2>Motion Correction Plots</H2>"
    puts $report "<p><a href=\"mc_rot.gif\">rotation</a> - <a href=\"mc_trans.gif\">translation</a>
    "
}

#}}}
	#{{{ BET

if { $fmri(bet_yn) } {
    feat4:exec "${FSLDIR}/bin/betfunc $funcdata ${FD}/prefiltered_func_data_bet"
    set funcdata ${FD}/prefiltered_func_data_bet
}

#}}}
	#{{{ filtering

set thecommand "${FSLDIR}/bin/ip ${funcdata} ${FD}/filtered_func_data $fmri(brain_thresh)"

#{{{ Spatial filtering

if { $fmri(smooth) > 0.01 } {
    set thecommand "$thecommand -s [ expr $fmri(smooth) / 2.355 ]"
    set ps "$ps; spatial smoothing using a Gaussian kernel of FWHM $fmri(smooth)mm"
}

#}}}
#{{{ Masking and global intensity normalization

set thecommand "$thecommand -m ${FD}/mask"

set global_mean 10000

if { $fmri(norm_yn)} {
    set thecommand "$thecommand -i $global_mean"
    set ps "$ps; global (volumetric) multiplicative mean intensity renormalization"
} else {
    set thecommand "$thecommand -I $global_mean"
    set ps "$ps; mean-based intensity normalisation of all volumes by the same factor"
}

#}}}
#{{{ Temporal filtering

if { $fmri(temphp_yn) || $fmri(templp_yn) } {

    set hp_sigma_vol -1
    if { $fmri(temphp_yn) } {
	set hp_sigma_sec [ expr $fmri(paradigm_hp) / 2.0 ]
	set hp_sigma_vol [ expr $hp_sigma_sec / $fmri(tr) ]
	set ps "$ps; nonlinear highpass temporal filtering (Gaussian-weighted LSF straight line fitting, with sigma=${hp_sigma_sec}s)"
    }

    set lp_sigma_vol -1
    if { $fmri(templp_yn) } {
	set lp_sigma_sec 2.8
	set lp_sigma_vol [ expr $lp_sigma_sec / $fmri(tr) ]
	set ps "$ps; Gaussian lowpass temporal filtering HWHM ${lp_sigma_sec}s"
    }

    set thecommand "$thecommand -t $hp_sigma_vol $lp_sigma_vol"

}

#}}}
#{{{ Call ip

set ps "$ps."

puts $logout "$thecommand"

set absbrainthresh [ exec sh -c $thecommand | grep THRESHOLD | awk {{ print $2 }} - ]

#}}}

#}}}

    } else {
	exec cp ${funcdata}.hdr ${FD}/filtered_func_data.hdr
	exec cp ${funcdata}.img ${FD}/filtered_func_data.img
    }
}

#{{{ make binary mask if it hasn't already been made

if { ! [ file exists ${FD}/mask.hdr ] } {
    set thecommand "${FSLDIR}/bin/ip ${funcdata} ${FD}/prefiltered_func_data $fmri(brain_thresh) -m ${FD}/mask"
    puts $logout "$thecommand"
    set absbrainthresh [ exec sh -c $thecommand | grep THRESHOLD | awk {{ print $2 }} - ]
}

#}}}

feat4:exec "rm -rf ${FD}/prefiltered_func_data*"

#}}}
		#{{{ stats

if { $fmri(stats_yn) || $fmri(analysis) == 4 } {

    if { $fmri(paradigm_type) == 3 } {
	if { $fmri(analysis) != 4 } {
	    #{{{ FILM

if { $fmri(type) == 2 } {
    set film_opts "-sa -ms 5"
} else {
    set film_opts "-gl"
}

feat4:exec "$FSLDIR/bin/film_gls -rn ${FD}/stats $film_opts -sp $FSLDIR/bin/susan_smooth ${FD}/filtered_func_data ${FD}/design.mat $absbrainthresh"

if { $fmri(dim3) > 2 } {
    feat4:exec "$FSLDIR/bin/smoothest -d [ exec cat ${FD}/stats/dof ] -m ${FD}/mask -r ${FD}/stats/res4d > ${FD}/stats/smoothness"
}

feat4:exec "rm -f ${FD}/stats/res4d*"

#}}}
	}
	#{{{ run contrasts

set FTESTS ""
if { [ file exists ${FD}/design.fts ] } {
    set FTESTS "-f ${FD}/design.fts"
}

feat4:exec "$FSLDIR/bin/contrast_mgr $FTESTS ${FD}/stats ${FD}/design.con"

set ps "$ps Statistical analysis was carried out using FILM (FMRIB's Improved Linear Model) with local autocorrelation correction \[Woolrich 2000\]."
set rs "$rs\[Woolrich 2000\] M.W. Woolrich, B.D. Ripley, J.M. Brady and S.M. Smith. Temporal Autocorrelation in Univariate Linear Modelling of FMRI Data. NeuroImage in submission 2001.<br>
"

#}}}
    } else {
	#{{{ IRVA

set fmri(ncon) 1

if { $fmri(analysis) != 4 } {

    exec /bin/mkdir ${FD}/stats

    set dofout [ open ${FD}/stats/dof "w" ]
    puts $dofout $total_volumes
    close $dofout

    if { $fmri(mf_type) == 1 } {
	set thecommand "${FSLDIR}/bin/irva ${FD}/filtered_func_data ${FD}/stats/fstat1 ${FD}/stats/zstat1 $fmri(brain_thresh) 0 [ expr int($fmri(mf_cycle)) ]"
    } else {
	set mf_events [ exec wc -l $fmri(mf_file) | awk {{ print $1 }} - ]
	set thecommand "${FSLDIR}/bin/irvanc ${FD}/filtered_func_data ${FD}/stats/fstat1 ${FD}/stats/zstat1 $fmri(brain_thresh) $mf_events [ expr int($fmri(mf_cycle)) ] $fmri(mf_file)"
    }

    feat4:exec $thecommand

} else {
    exec /bin/cp -f ${OldFD}/stats/zstat1.hdr ${FD}/stats/zstat1.hdr
    exec /bin/cp -f ${OldFD}/stats/zstat1.img ${FD}/stats/zstat1.img
}   

#no idea why this was here.....
#set fmri(paradigm_type) 1

set ps "$ps Statistical analysis was carried out using the IRVA (Inter-Repetition Variance Analysis) method \[Clare 1999\]."
set rs "$rs\[Clare 1999\] S. Clare, M. Humberstone, J. Hykin, L. Blumhardt, R. Bowtell and P. Morris. Detecting Activations in Event-Related fMRI Using Analysis of Variance. Magnetic Resonance in Medicine 1999:42(1117-1122).<br>
"

#}}}
    }
}

#}}}
		#{{{ poststats
		
		if { $fmri(poststats_yn) } {
		    feat4:poststats 0
		}

#}}}
		#{{{ time series plots and report output for each contrast

if { $fmri(stats_yn) || $fmri(poststats_yn) } {

    #{{{ run tsplot

set fmrifile ${FD}/filtered_func_data
if { ! [ file exists ${fmrifile}.hdr ] } {
    set fmrifile [ string trimright [ file root ${FD} ] + ]
}

feat4:exec "exec ${FSLDIR}/bin/tsplot ${FD} -f $fmrifile"

#}}}

    if { [ file exists ${FD}/tsplot_zstat1.gif ] } {
	puts $report "<hr><H2>Time series plots</H2><p>"
	#{{{ t stats

	for { set i 1 } { $i <= $fmri(ncon) } { incr i 1 } {
	    if { $fmri(conpic.$i) == 1 } {
		if { $fmri(paradigm_type) == 3 } {
		    puts $report "<a href=\"tsplot_zstat${i}p.gif\"><IMG BORDER=0 SRC=\"tsplot_zstat${i}.gif\"></a><br>
		    "
		    if { [ file exists ${FD}/tsplotc_zstat${i}.gif ] } {
			puts $report "<a href=\"tsplotc_zstat${i}p.gif\"><IMG BORDER=0 SRC=\"tsplotc_zstat${i}.gif\"></a><br>
			"
		    }
		} else {
		    puts $report "<IMG BORDER=0 SRC=\"tsplot_zstat${i}.gif\"><br>
		    "
		    if { [ file exists ${FD}/tsplotc_zstat${i}.gif ] } {
			puts $report "<IMG BORDER=0 SRC=\"tsplotc_zstat${i}.gif\"><br>
			"
		    }
		}
	    }
	}

#}}}
	#{{{ f stats

	for { set i 1 } { $i <= $fmri(nftests) } { incr i 1 } {

	    if { [ file exists ${FD}/tsplot_zfstat${i}.gif ] } {

		puts $report "<IMG BORDER=0 SRC=\"tsplot_zfstat${i}.gif\"><br>
		"
		if { [ file exists ${FD}/tsplotc_zfstat${i}.gif ] } {
		    puts $report "<IMG BORDER=0 SRC=\"tsplotc_zfstat${i}.gif\"><br>
		    "
		}
	    }
	}

#}}}
    }
}

#}}}
		#{{{ clean up

set feat_files($session) $FD

#}}}
	    } else {
		#{{{ write starts of log and report files

set FD $feat_files($session)
catch { exec /bin/cp ${fsfroot}.fsf ${FD}/flirtdesign.fsf }
set fsfroot ${FD}/flirtdesign

set LOGFILE    ${FD}/flirtreport.log
set REPORTFILE ${FD}/flirtreport.html
set logout [ open $LOGFILE "w" ]
set report [ open $REPORTFILE "w" ]
fconfigure $logout -buffering none
fconfigure $report -buffering none

set tailpid [ exec tail -f $LOGFILE & ]
catch { exec sh -c "sleep 36000 ; kill -9 $tailpid" & } errmsg

puts $logout "Started FEAT at $thedate on $HOSTNAME"

puts $report "<HTML>

<TITLE>FLIRT Registration Report</TITLE>

<BODY BACKGROUND=\"file:${FSLDIR}/doc/images/fsl-bg\">

<hr><CENTER>
<H1>FLIRT Registration Report</H1>
$REPORTFILE<br>
for FEAT session<br>
<a href=\"${FD}/report.html\">${FD}</a><br>
$thedate
</CENTER>
"

set ps "<hr><H2>Analysis methods</H2>
<p>Analysis was carried out using FEAT, the FMRIB Easy Analysis Tool."

set rs "<p><b>References</b><br>
"

#}}}
	    }
	    #{{{ registration

# could move to start and run both in parallel 

if { $fmri(reg_yn) } {

    #{{{ report output

puts $report "<hr><H2>Registration results</H2>"

set ps "$ps Registration to high resolution and/or standard images was carried out using FLIRT \[Jenkinson 2001\]."
set rs "$rs\[Jenkinson 2001\] M. Jenkinson and S.M. Smith. A Global Optimisation Method for Robust Affine Registration of Brain Images. Medical Image Analysis 2001:5(2)(143-156).<br>
"

#}}}
    #{{{ short names for different files

set highres ""
if { $fmri(reghighres_yn) } {
    set highres $highres_files($session)
}

set ef ${FD}/example_func
set hr [ file rootname $highres ]
set st [ file rootname $fmri(regstandard) ]
set ef2hr ${FD}/example_func2highres
set hr2ef ${FD}/highres2example_func
set hr2st ${FD}/highres2standard
set st2hr ${FD}/standard2highres
set ef2st ${FD}/example_func2standard
set st2ef ${FD}/standard2example_func
set sl ${FD}/sliceroutput

#}}}
    #{{{ check FOV and set DOF

set MINFOV 120
set DOF 12

set xfov [ expr abs([ exec $FSLDIR/bin/avwval ${ef} pixdim1 ] * [ exec $FSLDIR/bin/avwval ${ef} dim1 ]) ]
set yfov [ expr abs([ exec $FSLDIR/bin/avwval ${ef} pixdim2 ] * [ exec $FSLDIR/bin/avwval ${ef} dim2 ]) ]
set zfov [ expr abs([ exec $FSLDIR/bin/avwval ${ef} pixdim3 ] * [ exec $FSLDIR/bin/avwval ${ef} dim3 ]) ]

puts $logout "Field of view of functional image = ($xfov,$yfov,$zfov)"

if { $xfov < $MINFOV || $yfov < $MINFOV || $zfov < $MINFOV } {
    puts $logout "Thus reducing functional-structural registration DOF to 7"
    set DOF 7
}

#}}}
    #{{{ func -> highres

if { $fmri(reghighres_yn) } {

    if { [file exists ${FD}/highres.hdr] } {
	exec rm -f ${FD}/highres.hdr
    }
    if { [file exists ${FD}/highres.img] } {
	exec rm -f ${FD}/highres.img
    }
    exec ln -f -s ${hr}.hdr ${FD}/highres.hdr
    exec ln -f -s ${hr}.img ${FD}/highres.img
    
    feat4:exec "$CLUSTERRSH ${FSLDIR}/bin/flirt -ref $hr -in $ef -out $ef2hr -omedx ${ef2hr}.xfm -omat ${ef2hr}.mat -dof $DOF -bins 256 -cost corratio -anglerep euler -nosave -searchrx -90 90 -searchry -90 90 -searchrz -90 90"

    feat4:exec "${FSLDIR}/bin/convert_xfm -ref $hr -in $ef -inverse -omedx ${hr2ef}.xfm -omat ${hr2ef}.mat ${ef2hr}.mat"

    feat4:exec "${FSLDIR}/bin/slicer $ef2hr $hr -s 0.5 -x 0.4 ${sl}a -x 0.5 ${sl}b -x 0.6 ${sl}c -y 0.4 ${sl}d -y 0.5 ${sl}e -y 0.6 ${sl}f -z 0.4 ${sl}g -z 0.5 ${sl}h -z 0.6 ${sl}i ; ${FSLDIR}/bin/convert -colors 100 +append ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i ${ef2hr}.gif ; /bin/rm -f ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i"

    puts $report "<p>FMRI (underlying image) to structural image (red lines)<br>
    <a href=\"example_func2highres.mat\"><IMG BORDER=0 SRC=\"example_func2highres.gif\"></a>
    "
}

#}}}
    #{{{ standard

if { $fmri(regstandard_yn) } {

    if { [file exists ${FD}/standard.hdr] } {
	exec rm -f ${FD}/standard.hdr
    }
    if { [file exists ${FD}/standard.img] } {
	exec rm -f ${FD}/standard.img
    }
    exec ln -f -s ${st}.hdr ${FD}/standard.hdr
    exec ln -f -s ${st}.img ${FD}/standard.img

    if { $fmri(reghighres_yn) } {
	feat4:exec "$CLUSTERRSH ${FSLDIR}/bin/flirt -ref $st -in $hr -out $hr2st -omedx ${hr2st}.xfm -omat ${hr2st}.mat -dof 12 -bins 256 -cost corratio -anglerep euler -nosave -searchrx -90 90 -searchry -90 90 -searchrz -90 90"
	TalairachMedxTransform ${hr2st}.xfm

	feat4:exec "${FSLDIR}/bin/convert_xfm -ref $st -in $hr -inverse -omedx ${st2hr}.xfm -omat ${st2hr}.mat ${hr2st}.mat"

	feat4:exec "${FSLDIR}/bin/slicer $hr2st $st -s 1 -x 0.4 ${sl}a -x 0.5 ${sl}b -x 0.6 ${sl}c -y 0.4 ${sl}d -y 0.5 ${sl}e -y 0.6 ${sl}f -z 0.4 ${sl}g -z 0.5 ${sl}h -z 0.6 ${sl}i ; ${FSLDIR}/bin/convert -colors 100 +append ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i ${hr2st}.gif ; /bin/rm -f ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i"

	puts $report "<p>Structural image (underlying image) to standard image (red lines)<br>
	<a href=\"highres2standard.mat\"><IMG BORDER=0 SRC=\"highres2standard.gif\"></a>
	"

	feat4:exec "${FSLDIR}/bin/convert_xfm -ref $st -in $ef -omedx ${ef2st}.xfm -omat ${ef2st}.mat -concat ${hr2st}.xfm -middlevol $hr ${ef2hr}.xfm"
	TalairachMedxTransform ${ef2st}.xfm
	feat4:exec "${FSLDIR}/bin/convert_xfm -ref $st -in $ef -inverse -omedx ${st2ef}.xfm -omat ${st2ef}.mat ${ef2st}.mat"

	feat4:exec "$CLUSTERRSH ${FSLDIR}/bin/flirt -ref $st -in $ef -out $ef2st -init ${ef2st}.mat -applyxfm"
	
	feat4:exec "${FSLDIR}/bin/slicer $ef2st $st -s 1 -x 0.4 ${sl}a -x 0.5 ${sl}b -x 0.6 ${sl}c -y 0.4 ${sl}d -y 0.5 ${sl}e -y 0.6 ${sl}f -z 0.4 ${sl}g -z 0.5 ${sl}h -z 0.6 ${sl}i ; ${FSLDIR}/bin/convert -colors 100 +append ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i ${ef2st}.gif ; /bin/rm -f ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i"

	puts $report "<p>FMRI (underlying image) to standard image (red lines)<br>
	<a href=\"example_func2standard.mat\"><IMG BORDER=0 SRC=\"example_func2standard.gif\"></a>
	"
    } else {
	feat4:exec "$CLUSTERRSH ${FSLDIR}/bin/flirt -ref $st -in $ef -out $ef2st -omedx ${ef2st}.xfm -omat ${ef2st}.mat -dof $DOF -bins 256 -cost corratio -anglerep euler -nosave -searchrx -90 90 -searchry -90 90 -searchrz -90 90"
	TalairachMedxTransform ${ef2st}.xfm

	feat4:exec "${FSLDIR}/bin/convert_xfm -ref $st -in $ef -inverse -omedx ${st2ef}.xfm -omat ${st2ef}.mat ${ef2st}.mat"
	
	feat4:exec "${FSLDIR}/bin/slicer $ef2st $st -s 1 -x 0.4 ${sl}a -x 0.5 ${sl}b -x 0.6 ${sl}c -y 0.4 ${sl}d -y 0.5 ${sl}e -y 0.6 ${sl}f -z 0.4 ${sl}g -z 0.5 ${sl}h -z 0.6 ${sl}i ; ${FSLDIR}/bin/convert -colors 100 +append ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i ${ef2st}.gif ; /bin/rm -f ${sl}a ${sl}b ${sl}c ${sl}d ${sl}e ${sl}f ${sl}g ${sl}h ${sl}i"
	
	puts $report "<p>FMRI example image to standard image<br>
	<a href=\"example_func2standard.mat\"><IMG BORDER=0 SRC=\"example_func2standard.gif\"></a>
	"
    }
}

#}}}

    feat4:exec "/bin/rm -f ${ef2hr}.hdr ${ef2hr}.img ${hr2st}.hdr ${hr2st}.img ${ef2st}.hdr ${ef2st}.img"

} else {

    if { $fmri(analysis) == 4 && [ file exists ${OldFD}/example_func2standard.mat ] } {
	puts $report "<hr>To see registration look <a href=\"${OldFD}/report.html\">here</a>."
    } else {
	puts $report "<hr>Registration was not carried out initially - later registration results will appear <a href=\"flirtreport.html\">here</a>."
    }

}

#}}}
	    #{{{ Talairach space clusters

if { [ file exists ${FD}/example_func2standard.mat ] } {

    set tmp_z_thresh    $fmri(z_thresh)
    set tmp_prob_thresh $fmri(prob_thresh)

    if { $fmri(analysis) == 0 } {
	set tmp_z_thresh    [ exec grep "set fmri(z_thresh)" ${FD}/design.fsf | awk {{ print $3 }} - ]
	set tmp_prob_thresh [ exec grep "set fmri(prob_thresh)" ${FD}/design.fsf | awk {{ print $3 }} - ]
    }

    foreach statstypes { zstat zfstat } {

	set clusterlist [ glob -nocomplain ${FD}/thresh_${statstypes}*.hdr ]

	foreach cluster $clusterlist {

	    set i [ string trimleft [ file tail [ file root $cluster ] ] thresh_${statstypes} ]

	    set COPE ""
	    if { [ file exists ${FD}/stats/cope$i ] } {
		set COPE "-c ${FD}/stats/cope$i"
	    }

	    if { [ file exists ${FD}/stats/smoothness ] } {
		set DLH     [exec grep DLH    ${FD}/stats/smoothness | awk {{ print $2 }} - ]
		set VOLUME  [exec grep VOLUME ${FD}/stats/smoothness | awk {{ print $2 }} - ]
		set RESELS  [exec grep RESELS ${FD}/stats/smoothness | awk {{ print $2 }} - ]
		set NRESELS [ expr int ( $VOLUME / $RESELS ) ]
	    } else {
		feat4:exec "$FSLDIR/bin/smoothest -m ${FD}/mask -z ${FD}/stats/${statstypes}$i > ${FD}/stats/${statstypes}$i.smoothness"
	    
		set DLH     [exec grep DLH    ${FD}/stats/${statstypes}${i}.smoothness | awk {{ print $2 }} - ]
		set VOLUME  [exec grep VOLUME ${FD}/stats/${statstypes}${i}.smoothness | awk {{ print $2 }} - ]
		set RESELS  [exec grep RESELS ${FD}/stats/${statstypes}${i}.smoothness | awk {{ print $2 }} - ]
		set NRESELS [ expr int ( $VOLUME / $RESELS ) ]
	    
		puts $logout "DLH=$DLH VOLUME=$VOLUME RESELS=$RESELS NRESELS=$NRESELS
		"
	    }

	    feat4:exec "$FSLDIR/bin/cluster -i ${FD}/thresh_${statstypes}${i} ${COPE} -t $tmp_z_thresh -p $tmp_prob_thresh -d $DLH --volume=$VOLUME -x ${FD}/example_func2standard.mat --talvol=${FD}/standard --mm --connectivity=[ feat4:connectivity ${FD}/thresh_${statstypes}${i} ] > ${FD}/cluster_${statstypes}${i}_tal.txt"
	    feat4:exec "$FSLDIR/bin/cluster2html $FD cluster_${statstypes}${i} -tal"
	}
    }
}

#}}}
	    #{{{ timing and final outputs

puts $report "$ps

$rs

"

if { [ file exists ${FD}/design.mat ] } {
    puts $report "<p><p><b>Design matrix</b><br><a href=\"design.mat\"><IMG BORDER=0 SRC=\"design.gif\"></a>
    "

    if { [ file exists ${FD}/design_cov.gif ] } {
	puts $report "<p><p><b>Covariance matrix</b><br><IMG BORDER=0 SRC=\"design_cov.gif\">
	"
    }
}

puts $report "<HR><FONT SIZE=1>This page produced automatically by FEAT $VERSION - a part of <A HREF=\"http://www.fmrib.ox.ac.uk/fsl\">FSL - FMRIB's Software Library</A>.</FONT>

</BODY></HTML>
"

puts $logout "Finished FEAT at [ exec date ]
To view the FEAT report point your web browser at
$REPORTFILE
"

close $logout
close $report

after 2000

catch { exec kill -9 $tailpid } errmsg

#}}}
	}
    }

    if { $fmri(group_yn) } {
	#{{{ get group file name and setup folder details

set FD [ new_filename [ file rootname $feat_files(1) ].gfeat ]

if { ! [ file writable [ file dirname $FD ] ] } {
    set FD [ new_filename ${HOME}/[ file rootname [ file tail $feat_files(1) ] ].gfeat ]
}

exec /bin/mkdir $FD

# we write rather than copy, because we may have overwritten
# avw filenames with .feat directory names
feat4:write -1 0 ${FD}/design.fsf

exec /bin/mkdir ${FD}/stats

catch { exec cp $feat_files(1)/design.gif $feat_files(1)/design.mat $FD }

#}}}
	#{{{ write starts of log and report files

set LOGFILE    ${FD}/report.log
set REPORTFILE ${FD}/report.html
set logout [ open $LOGFILE "w" ]
set report [ open $REPORTFILE "w" ]
fconfigure $logout -buffering none
fconfigure $report -buffering none

set tailpid [ exec tail -f $LOGFILE & ]
catch { exec sh -c "sleep 36000 ; kill -9 $tailpid" & } errmsg

set thedate [ exec date ]

puts $logout "Started Group FEAT at $thedate on $HOSTNAME"

puts $report "<HTML>

<TITLE>Group FEAT Report</TITLE>

<BODY BACKGROUND=\"file:${FSLDIR}/doc/images/fsl-bg\">

<hr><CENTER>
<H1>Group FEAT Report</H1>
$REPORTFILE
$thedate
</CENTER>
"

set grouptype "fixed and random effects"
if { $fmri(paradigm_type) == 4 } {
    set grouptype "simplified fixed effects"
}

set ps "<hr><H2>Analysis methods</H2>
<p>Group ($grouptype) analysis was carried out using FEAT, the FMRIB Easy Analysis Tool."

set rs "<p><b>References</b><br>
"

#}}}
	#{{{ links to first-level reports

puts $report "<hr><H2>First-level reports</H2>"

if { $fmri(ngroupa) > 0 } {
    puts $report "<b>Group A</b> (control):<br>
"
}

for { set i 1 } { $i <= $fmri(multiple) } { incr i 1 } {

    if { $fmri(ngroupa)>0 && $i==$fmri(ngroupa)+1 } {
	puts $report "<b>Group B</b> (test):<br>
	"
    }

    puts $report "FEAT report ${i}: <A HREF=\"$feat_files($i)/report.html\">$feat_files($i)/report.html</A><br>
    "

}

#}}}
	#{{{ do the group stats

set featdirlist ""
for { set i 1 } { $i <= $fmri(multiple) } { incr i 1 } {
    set featdirlist "$featdirlist $feat_files($i)"
}

feat4:exec "exec $CLUSTERRSH ${FSLDIR}/bin/gfeat $fmri(ngroupa) [ expr $fmri(multiple) - $fmri(ngroupa) ] $featdirlist ${FD}/stats"

# fix headers
if { [ file exists ${FD}/stats/.hdr ] } {
    exec sh -c "cd $FD ; for ff in stats/fixed*.hdr stats/random*.hdr ; do cp -f stats/.hdr \$ff ; done"
}

feat4:exec "cd $FD/stats ; mv mask.* mean_highres.* .."

if { ! [ file exists ${FD}/mean_highres.hdr ] } {
    exec sh -c "cp [ file rootname $fmri(regstandard) ].hdr $FD/mean_highres.hdr"
    exec sh -c "cp [ file rootname $fmri(regstandard) ].img $FD/mean_highres.img"
}

#}}}
	#{{{ poststats

if { $fmri(poststats_yn) } {
    feat4:poststats 1
}

# fix headers
# don't think I need this any more
#exec sh -c " cd $FD ; for ff in *.hdr ; do ${FSLDIR}/bin/avwcpgeom stats/.hdr \$ff ; done "

#}}}
	#{{{ timing and final outputs

puts $report "$ps

$rs

"

if { [ file exists ${FD}/design.gif ] } {
    puts $report "<p><p><b>Design matrix</b><br><a href=\"$feat_files(1)/design.mat\"><IMG BORDER=0 SRC=\"$feat_files(1)/design.gif\"></a>
    "
}

puts $report "<HR><FONT SIZE=1>This page produced automatically by FEAT $VERSION - a part of <A HREF=\"http://www.fmrib.ox.ac.uk/fsl\">FSL - FMRIB's Software Library</A>.</FONT>

</BODY></HTML>
"

puts $logout "Finished FEAT at [ exec date ]
To view the Group FEAT report point your web browser at
$REPORTFILE
"

close $logout
close $report

after 2000

catch { exec kill -9 $tailpid } errmsg

#}}}
    }
}

#}}}
#{{{ call GUI and wait

if { ! [ info exists INGUI ] } {
    wm withdraw .
    feat4 .r
    tkwait window .r
}

#}}}
