/*  ContrastMgrOptions.cc

    Mark Woolrich, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

#define WANT_STREAM
#define WANT_MATH

#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>
#include "newmatap.h"
#include "newmatio.h"
#include "ContrastMgrOptions.h"

using namespace NEWMAT;
namespace FILM {

ContrastMgrOptions* ContrastMgrOptions::gopt = NULL;

void ContrastMgrOptions::parse_command_line(int argc, char** argv, Log& logger)
{
  if(argc<2){
    print_usage(argc,argv);
    exit(1);
  }
  
  int inp = 1;
  int n=1;
  string arg;
  char first;
  
  while (n<argc) {
    arg=argv[n];

    if (arg.size()<1) { n++; continue; }
    first = arg[0];
    if (first!='-') {
      if(inp == 1)
	gopt->datadir = arg;
      else if(inp == 2)
	gopt->contrastfname = arg;      
      else
	{
	  cerr << "Mismatched argument " << arg << endl;
	  break;
	}
      n++;
      inp++;
      continue;
    }
    
    // put options without arguments here
    if ( arg == "-help" ) {
      print_usage(argc,argv);
      exit(0);
    } else if ( arg == "-v" ) {
      gopt->verbose = true;
      n++;
      continue;
    } else if ( arg == "-d" ) {
      gopt->debug = true;
      n++;
      continue;
    }

    if ( arg == "-cope") {
      gopt->copenumber = atoi(argv[n+1]);
      n+=2;
      continue;
    } 
    else if ( arg == "-suffix") {
      gopt->suffix = argv[n+1];
      n+=2;
      continue;
    } 
    else if (arg == "-f") {
      gopt->fcontrastfname = argv[n+1];
      n+=2;
      continue;
    } 

    if (n+1>=argc) 
      { 
	cerr << "Lacking argument to option " << arg << endl;
	break; 
      }
  }  // while (n<argc)

  logger.setDir(gopt->datadir,"cmlogfile");
  for(int a = 0; a < argc; a++)
    logger.str() << argv[a] << " ";
  logger.str() << endl << "---------------------------------------------" << endl << endl;

  if (gopt->datadir.size()<1) {
    print_usage(argc,argv);
    throw Exception("Data directory not found");
  }
  if (gopt->contrastfname.size()<1) {
    print_usage(argc,argv);
    throw Exception("Contrast filename not found");
  }
 
}

void ContrastMgrOptions::print_usage(int argc, char *argv[])
{

  cout << "Usage: " << argv[0] << " [options] <datadir> <tcontrastfile>\n\n"
       << "  Available options are:\n"
       << "        -v                                 (outputs full data)\n"
       << "        -cope <num>                        (contrast number to start labelling copes from, default is 1)\n"
       << "        -suffix <string>                   (suffix to put on the end of the cope filename before the contrast number, default is nothing)\n"
       << "        -f <fcontrastfile>                 (fcontrast file)\n"
       << "        -d                                 (debug)\n"
       << "        -help\n\n";
}

#ifndef NO_NAMESPACE
}
#endif
