/*  FilmOlsOptions.h

    Mark Woolrich, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

#if !defined(__FilmOlsOptions_h)
#define __FilmOlsOptions_h

#include <string>
#include <math.h>
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>

namespace FILM {

class FilmOlsOptions {
 public:
  static FilmOlsOptions& getInstance();
  ~FilmOlsOptions() { delete gopt; }

  string inputfname;
  string paradigmfname;
  string datadir;
  string epifname;
  string susanpath;

  bool verbose;
  bool detrend;
  bool smoothACEst;
  bool fitAutoRegressiveModel;
  bool globalEst;
  bool prewhiten;
  bool noest;

  int trimscans;
  int thresh;
  int ms;
  int epith;
  short maxshort;

  void parse_command_line(int argc, char** argv, ofstream& logfile);

 private:
  FilmOlsOptions();
  
  const FilmOlsOptions& operator=(FilmOlsOptions&);
  FilmOlsOptions(FilmOlsOptions&);
      
  static FilmOlsOptions* gopt;

  void print_usage(int argc, char *argv[]);
  
};

inline FilmOlsOptions& FilmOlsOptions::getInstance(){
  if(gopt == NULL)
    gopt = new FilmOlsOptions();
  
  return *gopt;
}

inline FilmOlsOptions::FilmOlsOptions()
{
  // set up defaults
  datadir = "results";  
  epifname = "epivolume";
  inputfname = "";  
  paradigmfname = "";
  susanpath = "/usr/local/fsl/bin/susan_smooth";
  
  thresh = 0;
  ms = 4;
  epith = 100;
  maxshort = 32000;
  fitAutoRegressiveModel = false;

  prewhiten = false;
  smoothACEst = false;
  verbose = false;
  detrend = false;
  globalEst = false;
  noest = false;
}

}

#endif









