/*  MELODIC - Multivariate exploratory linear optimized decomposition into 
              independent components
    
    melodicica.h - ICA estimation

    Christian F. Beckmann, FMRIB Image Analysis Group
    
    Copyright (C) 1999-2001 University of Oxford */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */


#ifndef __MELODICICA_h
#define __MELODICICA_h
#include "newimageall.h"
#include "Log.h"
#include "melodicpca.h"
#include "melodicoptions.h"
#include "melodicdata.h"
#include "melodic.h"
#include "newmatap.h"
#include "newmatio.h"

using namespace Utilities;
using namespace NEWIMAGE;

namespace Melodic{
  
  class MelodicICA
    {
    public:
      MelodicICA(MelodicData &pmelodat, MelodicOptions &popts, Log &plogger):  
	melodat(pmelodat),
	opts(popts),
	logger(plogger)
	{		
	} 
      
      void perf_ica(const Matrix &Data);
      
      bool no_convergence;

    private:
      MelodicData &melodat;
      MelodicOptions &opts;
      Log &logger;

      int dim;
      int samples; 
      Matrix redUMM;

      void ica_fastica_symm(const Matrix &Data);
      void ica_fastica_defl(const Matrix &Data);
      void ica_maxent(const Matrix &Data);
      void ica_jade(const Matrix &Data);
      Matrix randm(const int dim1, const int dim2);

    };   
}

#endif
