/* {{{ Copyright */

/*  runavw - works out what the data type is from argv[1] and then calls the
    correct program on the basis of argv[0] and the data type

    Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

/* }}} */
/* {{{ includes */

#include "libss.h"
#include "libavw.h"
#include <unistd.h>

/* }}} */
/* {{{ main */

int main(int argc, char *argv[])
{
  /* {{{ variables */

  FILE *fd;
  struct dsr hdr;
  char thestring[10000], filename[10000], filename2[10000];
  int i, datatype, bitpix;
  static int DataTypeSizes[32] = {0,1,8,0,16,0,0,0,32,0,0,0,0,0,0,0,32};

/* }}} */

  sprintf(thestring,"%s",argv[0]);

  /* {{{ if full path name wasn't given for program, run "which" */

if (thestring[0]!='/')
{
  char thestring2[10000];
  int  pid=getpid();
  FILE *tmpfp;

  sprintf(thestring2,"which %s > /tmp/runavw%d",thestring,pid);
  system(thestring2);

  sprintf(thestring2,"/tmp/runavw%d",pid);
  tmpfp=fopen(thestring2,"r");
  fscanf(tmpfp,"%s",thestring);  
  fclose(tmpfp);

  sprintf(thestring2,"/bin/rm -f /tmp/runavw%d",pid);
  system(thestring2);
}

/* }}} */

  if (argc<2)
    sprintf(thestring,"%s_16SI",thestring); /* assume 16SI to get usage */
  else
    {
      strcpy(filename2,argv[1]);
      if ( (strcmp(".hdr",filename2+strlen(filename2)-4)==0) ||
	   (strcmp(".img",filename2+strlen(filename2)-4)==0) )
	filename2[strlen(filename2)-4]=0;

      sprintf(filename,"%s.hdr",filename2);
      if((fd=fopen(filename,"r"))==NULL)
	{
	  fprintf(stderr,"Can't open %s\n",filename);
	  exit(1);
	}
      fread(&hdr,1,sizeof(struct dsr),fd);

      if(hdr.hk.sizeof_hdr!=348)
	swap_hdr(&hdr);

      datatype=hdr.dime.datatype;
      bitpix=DataTypeSizes[datatype];
      sprintf(thestring,"%s_%d",thestring,bitpix);

      if (datatype==DT_UNSIGNED_CHAR) sprintf(thestring,"%sUI",thestring);
      if (datatype==DT_SIGNED_SHORT) sprintf(thestring,"%sSI",thestring);
      if (datatype==DT_SIGNED_INT) sprintf(thestring,"%sSI",thestring);
      if (datatype==DT_FLOAT) sprintf(thestring,"%sR",thestring);
    }

  if((fd=fopen(thestring,"r"))==NULL)
    {
      fprintf(stderr,"%s has not been compiled for the data type of %s\n",argv[0],argv[1]);
      exit(1);
    }
  fclose(fd);

  sprintf(thestring,"%s %s",thestring,filename2);

  for(i=2; i<argc; i++)
    {
      sprintf(thestring,"%s %s",thestring,argv[i]);
    }

  return system(thestring);
}

/* }}} */
