
/*
  im->max and im->min represent the outer values of the *histogram*
    NOT the actual intensity data
  size represents the number of points in the normal histogram range

*/

void find_thresholds (image_struct *im, double fraction)
{
  int hist[HISTOGRAM_BINS], top_bin, bottom_bin, count,
    size=im->t*im->z*im->y*im->x, pass=1,
    lowest_bin=0, highest_bin=HISTOGRAM_BINS-1;

  while ( (pass==1) ||
	  ( (im->thresh98 - im->thresh2) < (im->max - im->min) / 10 ) ) 
    /* is the middle 96% of stuff too squashed up (i.e. in <10% of the bins) */

    {
      FDT tmpmin;

      if (pass>1) /* redo histogram with new min and max */
	{
	  /* increase range slightly from the 2-98% range found */
	  bottom_bin=MAX(bottom_bin-1,0);           
	  top_bin=MIN(top_bin+1,HISTOGRAM_BINS-1);

	  /* now set new min and max on the basis of this new range */
	  tmpmin = (FDT)( im->min +
			  ((double)bottom_bin/(double)(HISTOGRAM_BINS-1))*((double)im->max-(double)im->min)
			  );
	  im->max = (FDT)( im->min +
			   ((double)top_bin/(double)(HISTOGRAM_BINS-1))*((double)im->max-(double)im->min)
			   );
	  im->min=tmpmin;
	}

      if (pass==10)  /* give up and revert to full range _but_ ignore end
			bins */
	{
	  im->min = im->max = MAX_PASSES;
	  size=size-(hist[lowest_bin]+hist[highest_bin]);
	  lowest_bin++;
	  highest_bin--;
	}

      find_histogram (im,hist,HISTOGRAM_BINS);

      for(count=0, bottom_bin=lowest_bin; count<size/50; bottom_bin++)
	count+=hist[bottom_bin];
      bottom_bin--;
      im->thresh2 = (FDT)( im->min +
			   ((double)bottom_bin/(double)(HISTOGRAM_BINS-1))*((double)im->max-(double)im->min)
			   );

      for(count=0, top_bin=highest_bin; count<size/50; top_bin--)
	count+=hist[top_bin];
      top_bin++;
      im->thresh98 = (FDT)( im->min +
			    ((double)top_bin/(double)(HISTOGRAM_BINS-1))*((double)im->max-(double)im->min)
			    );

      if (pass==MAX_PASSES)
	break;

      pass++;
    }


  im->thresh = (FDT) ( im->thresh2 + fraction * ( (double)im->thresh98 -
						  (double)im->thresh2 ) );
}



void find_histogram (image_struct *im, int *hist, int bins)
{
  FDT *imagep, *image=im->i;
  int i, size=im->t*im->z*im->y*im->x;

  /* zero histogram */
  for(i=0; i<bins; i++)
    hist[i]=0;

  if (im->min==im->max) /* set min and max if not already set */
    for(im->min=im->max=image[0], imagep=image, i=0; i<size; i++)
      {
	if (*imagep>im->max)
	  im->max=*imagep;
	if (*imagep<im->min)
	  im->min=*imagep;
	imagep++;
      }

  /* create histogram; the MIN is so that the maximum value falls in the
     last valid bin, not the (last+1) bin */
  for(imagep=image, i=0; i<size; i++)
    {
      hist[MAX(0, MIN(
		      (int)((((double)bins)*((double)(*imagep-im->min)))/((double)(im->max-im->min))),
		      bins-1) )]++;
      imagep++;
    }
}


