///////////////////////////////////////////////////////////////////////////
//
// @module imagereader.h |
//
// This file declare the interface class for the general image IO.
//
// Copyright 2000 Yongyue Zhang (yongyue@fmrib.ox.ac.uk), All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////

#ifndef __IMAGEREADER_H
#define __IMAGEREADER_H

#define MAX_PATH_LENGTH	100

#include "imagecore.h"

enum	ImageType {UNKNOWN=0, BMP=1, PGM=2, PPM=3, PS=10, AVW=200};
enum	CompressionMethod {COMnone=1, COMhuff=2, COMccittg3=3, COMccittg4=4, COMlzw=5, COMpackbits=32773};

bool WriteImage(const ZImageBase& refimage, std::fstream& file, bool reverse=false);

class ZImageReader
{
	char			m_FileName[MAX_PATH_LENGTH];
	ImageType		m_nImageType;
protected:
	int				m_nAVWVolume;					// which volume to read in a 4D AVW.

	CompressionMethod	m_eCompression;				//only for tiff format

	int				m_nQuality;
	bool			m_nStandardEncoding;

	int				m_nFrameRate;

	int				m_nRawW, m_nRawH, m_nRawD, m_nRawPixelType;
	bool			m_nRawLEndian, m_nRawColorPlanar;

public:

	ZImageReader ();
	ZImageReader (const char* fname, const char* ftype=NULL);

	static ImageType	Recognize(const char* fname, const char* ftype=NULL);
	ImageType	SetFile(const char* fname, const char* ftype=NULL);
	ImageType	SetFile(const char* fname, ImageType ntype);

	ZImageBase*	ReadFile();
	bool		ReadFile(ZImageBase& Image);
	bool		ReadFile(std::vector<ZImageBase*>& images);
	bool		WriteFile(const ZImageBase&, bool intensityscale=false);

	void		SetAVWVolume(int vol) { m_nAVWVolume = vol; }

	void		SetJPEGEncoding(int quality, bool standard_encoding=true) 
	{ m_nQuality = quality; m_nStandardEncoding = standard_encoding; }

	void		SetCompressionMethod(CompressionMethod method) {m_eCompression = method;}

	void		SetAVIFrameRate(int rate) { m_nFrameRate = rate; }

	void		SetRAWSpec(int width, int height, int depth, int pixeltype, bool byteorderlE, bool colorplanar) 
	{ 
		m_nRawW = width; m_nRawH = height; m_nRawD = depth; m_nRawPixelType = pixeltype; 
		m_nRawLEndian = byteorderlE; m_nRawColorPlanar = colorplanar;
	}
};

ZImageBase* CreateImageFromDIB(PBYTE data);

#endif
